VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "c32bppDIB"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Credits/Acknowledgements - Thanx goes to:
'   Paul Caton for his class on calling non VB-Friendly DLLs that use _cdecl calling convention
'       Used when calling non VB-friendly zLIB dll versions
'   Alfred Koppold for his PNG, VB-only, decompression routines.
'       Used when zLib & GDI+ not available
'   Carles P.V for his pvResize logic
'       Used when manually scaling images with NearestNeighbor or BiLinear interpolation
'   www.zlib.net for their free zLIB.dll, the standard DLL for compressing/decompressing PNGs
'       Without it, we'd be limited to GDI+ for creating PNGs
'   coders like you that provide constructive criticism to make this class better & more all-inclusive
'       Without your comments, this project probably would have died several versions/updates ago
' For most current updates/enhancements visit the following:
'   Visit http://www.planetsourcecode.com/vb/scripts/ShowCode.asp?txtCodeId=67466&lngWId=1
' To see a usercontrol applying a version of this class
'   AlphaImage Control. http://www.planetsourcecode.com/vb/scripts/ShowCode.asp?txtCodeId=68262&lngWId=1
'   Image List Control. http://www.planetsourcecode.com/vb/scripts/ShowCode.asp?txtCodeId=69621&lngWId=1

' = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
'                                    O V E R V I E W
' = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
' AlphaBlend API :: (msimg32.dll) , GDI+ API (gdiplus.dll)

' About 32bpp pre-multiplied RGB (pARGB) bitmaps, if you are not aware.
'   - These are used specifically for the AlphaBlend API & are GDI+ compatible
'   Advantages:
'       - Images can be per-pixel alpha blended
'       - Opacity can be simultaneously adjusted during rendering
'       - AlphaBlend does both BitBlt & StretchBlt for pARGB images.
'       - Speed: AlphaBlend & GDI+ are pretty quick APIs vs manual blending
'   Disadvantages:
'       - The original RGB values are permanently destroyed during pre-multiplying
'           -- Premultiplied formula: preMultipliedRed=(OriginalRed * Alpha) \ 255
'           -- There is no way to convert pARGB back to non-premultiplied RGB values
'              The formula would be: reconstructedRed=(preMultipliedRed * 255) \ Alpha.
'               but because of integer division when pre-multiplying, the result is very
'               close and if this should be premultiplied again & converted again, the
'               calculated colors can eventually be completely different than the original.
'               Fully opaque pixels & fully transparent pixels are not affected.
'           ** Note: When images are converted to PNG formats, removal of
'              premultiplication is performed to meet PNG specs.
'       - Displaying a pre-multiplied bitmap without AlphaBlend/GDI+ will not result in
'           the image being displayed as expected.
'       - Not ideal for saving due to its size: SizeOf= W x H x 4
'           -- better to save source image instead or compress the DIB bytes using favorite compression utility
'           -- with GDI+ or zLib, image can be converted to PNG for storage
'       - AlphaBlend API is not included/compatible with Win95, NT4 and lower
'       - AlphaBlend on Win9x systems can be buggy, especially when rendering to DIBs vs DDBs
' Note that GDI+ is standard on WinXP+, and can be used on Win98,ME,2K & on NT4 if SP6 is installed
' Download GDI+ from:
' http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdicpp/GDIPlus/GDIPlus.asp
' Download ZLib from: http://www.zlib.net

' ----------------------------------------------
' About Win95, Win98, WinME, NT3.5 & NT4 support
' ----------------------------------------------
' These routines may not honor AlphaBlend if it exists on those systems. Win98's version,
' for example, has several bugs that can crash the application when AlphaBlending to DIBs.
' NT4, NT3.5 & Win95 do not come with AlphaBlend and I do not have WinME to test with.
' Therefore, to support these older systems, the Render routine will alphablend manually
' regardless if the AlhpaBlend API exists on the older system or not. However, this can
' be overridden by you. See isAlphaBlendFriendly routine. Therefore, AlphaBlend is only
' reliable on Win2K and above. XP & above already have GDI+


' Class Purpose:
' ----------------------------------------------
' This class holds the 32bpp image. It also marshals any new image thru
' the battery of parsers to determine best method for converting the image
' to a 32bpp alpha-compatible image. It handles rendering, rotating, scaling,
' mirroring of DIBs using manual processes, AlphaBlend, and/or GDI+.

' The parser order is very important for fastest/best results...
' cPNGparser :: will convert PNG, all bit depths; aborts quickly if not PNG
' cGIFparser :: will convert non-transparent/transparent GIFs; aborts quickly
' cICOpraser :: will convert XP-Alpha, paletted, true color, & Vista PNG icons
'               -- can also convert most non-animated cursors
' cBMPparser :: will convert bitmaps, wmf/emf & jpgs

' As a last resort, when GDI+ exists, anything unable to be processed by the
' parsers (i.e., TIFFs) are sent to GDI+. If GDI+ can process the image, then
' the image will be converted, internally, to PNG to enable additional processing.

' The parsers are efficient. Most image formats have a magic number that give
'   a hint to what type of image the file/stream is. However, checks need to
'   be employed because non-image files could feasibly have those same magic
'   numbers. If the image is determined not to be one the parser is designed
'   to handle, the parser rejects it and the next parser takes over.  The
'   icon parser is slightly different because PNG files can be included into
'   a Vista ico file. When this occurs, the icon parser will pass off the
'   PNG format to the PNG parser automatically.
' And last but not least, the parsers have no advanced knowledge of the image
' format; as far as they are concerned, anything passed is just a byte array

' Class Organization:
' ----------------------------------------------
' Search the class for the words NEW SECTION
' The class routines are organized in the following sections:
'   Class Initialization & Termination Routines
'   Public Properties & Methods (almost 60 and growing)
'       Public Read-Only Properties
'       Public Methods
'       Class to Class Communication Methods
'   Local Support Functions
' ----------------------------------------------

' = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
'                                       CHANGE HISTORY
' = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
' Accompanying FAQ.rtf is updated with every change
' Last changed: 18 Nov 07. See change history within the FAQ file
' 26 Dec 06: First version
' = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

' No APIs are declared public. This is to prevent possibly, differently
' declared APIs, or different versions of the same API, from conflicting
' with any APIs you declared in your project. Same rule for UDTs.
' Note: I did take liberties, changing parameter types, in several APIs throughout

' Used to determine operating system
Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" (lpVersionInformation As Any) As Long
Private Type OSVERSIONINFOEX
   dwOSVersionInfoSize As Long
   dwMajorVersion As Long
   dwMinorVersion As Long
   dwBuildNumber As Long
   dwPlatformId As Long
   szCSDVersion As String * 128 ' up to here is OSVERSIONINFO vs EX
   wServicePackMajor As Integer ' 8 bytes larger than OSVERSIONINFO
   wServicePackMinor As Integer
   wSuiteMask As Integer
   wProductType As Byte
   wReserved As Byte
End Type

' APIs used to manage the 32bpp DIB
Private Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (ByRef Destination As Any, ByRef Source As Any, ByVal Length As Long)
Private Declare Sub FillMemory Lib "kernel32.dll" Alias "RtlFillMemory" (ByRef Destination As Any, ByVal Length As Long, ByVal Fill As Byte)
Private Declare Function CreateCompatibleDC Lib "gdi32.dll" (ByVal hDC As Long) As Long
Private Declare Function GetDC Lib "user32.dll" (ByVal hwnd As Long) As Long
Private Declare Function ReleaseDC Lib "user32.dll" (ByVal hwnd As Long, ByVal hDC As Long) As Long
Private Declare Function DeleteDC Lib "gdi32.dll" (ByVal hDC As Long) As Long
Private Declare Function SelectObject Lib "gdi32.dll" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32.dll" (ByVal hObject As Long) As Long
Private Declare Function CreateDIBSection Lib "gdi32.dll" (ByVal hDC As Long, ByRef pBitmapInfo As Any, ByVal un As Long, ByRef Pointer As Long, ByVal Handle As Long, ByVal dw As Long) As Long
Private Declare Function AlphaBlend Lib "msimg32.dll" (ByVal hdcDest As Long, ByVal nXOriginDest As Long, ByVal nYOriginDest As Long, ByVal nWidthDest As Long, ByVal nHeightDest As Long, ByVal hdcSrc As Long, ByVal nXOriginSrc As Long, ByVal nYOriginSrc As Long, ByVal nWidthSrc As Long, ByVal nHeightSrc As Long, ByVal lBlendFunction As Long) As Long
Private Declare Function SetStretchBltMode Lib "gdi32.dll" (ByVal hDC As Long, ByVal nStretchMode As Long) As Long
Private Declare Function GetObjectType Lib "gdi32.dll" (ByVal hgdiobj As Long) As Long
Private Declare Function GetCurrentObject Lib "gdi32.dll" (ByVal hDC As Long, ByVal uObjectType As Long) As Long
Private Declare Function GetIconInfo Lib "user32.dll" (ByVal hIcon As Long, ByRef piconinfo As ICONINFO) As Long
Private Declare Function BitBlt Lib "gdi32.dll" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Private Declare Function SetDIBitsToDevice Lib "gdi32.dll" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal dX As Long, ByVal dY As Long, ByVal SrcX As Long, ByVal SrcY As Long, ByVal Scan As Long, ByVal NumScans As Long, ByRef Bits As Any, ByRef BitsInfo As BITMAPINFO, ByVal wUsage As Long) As Long
Private Declare Function GetDIBits Lib "gdi32.dll" (ByVal aHDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, ByRef lpBits As Any, ByRef lpBI As BITMAPINFO, ByVal wUsage As Long) As Long
Private Declare Function OffsetRgn Lib "gdi32.dll" (ByVal hRgn As Long, ByVal X As Long, ByVal Y As Long) As Long
Private Const STRETCH_HALFTONE As Long = &H4&
Private Const OBJ_BITMAP As Long = &H7&
Private Const OBJ_METAFILE As Long = &H9&
Private Const OBJ_ENHMETAFILE As Long = &HD&

' APIs used to create files
Private Declare Function WriteFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToWrite As Long, lpNumberOfBytesWritten As Long, lpOverlapped As Any) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Private Declare Function GetFileSize Lib "kernel32.dll" (ByVal hFile As Long, ByRef lpFileSizeHigh As Long) As Long
Private Declare Function ReadFile Lib "kernel32.dll" (ByVal hFile As Long, ByRef lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, ByRef lpNumberOfBytesRead As Long, ByRef lpOverlapped As Any) As Long
Private Declare Function SetFilePointer Lib "kernel32.dll" (ByVal hFile As Long, ByVal lDistanceToMove As Long, ByRef lpDistanceToMoveHigh As Long, ByVal dwMoveMethod As Long) As Long
Private Const INVALID_HANDLE_VALUE = -1&

' ////////////////////////////////////////////////////////////////
' Unicode-capable Drag and Drop of file names with wide characters
' ////////////////////////////////////////////////////////////////
Private Declare Function DispCallFunc Lib "oleaut32" (ByVal pvInstance As Long, _
    ByVal offsetinVft As Long, ByVal CallConv As Long, ByVal retTYP As VbVarType, _
    ByVal paCNT As Long, ByRef paTypes As Integer, _
    ByRef paValues As Long, ByRef retVAR As Variant) As Long
Private Declare Function lstrlenW Lib "kernel32.dll" (lpString As Any) As Long
Private Declare Function GlobalFree Lib "kernel32.dll" (ByVal hMem As Long) As Long

' ////////////////////////////////////////////////////////////////
' Unicode-capable Pasting of file names with wide characters
' ////////////////////////////////////////////////////////////////
Private Declare Function DragQueryFile Lib "shell32.dll" Alias "DragQueryFileA" (ByVal hDrop As Long, ByVal UINT As Long, ByVal lpStr As String, ByVal ch As Long) As Long
Private Declare Function OpenClipboard Lib "user32.dll" (ByVal hwnd As Long) As Long
Private Declare Function GetClipboardData Lib "user32.dll" (ByVal wFormat As Long) As Long
Private Declare Function CloseClipboard Lib "user32.dll" () As Long
Private Type FORMATETC
    cfFormat As Long
    pDVTARGETDEVICE As Long
    dwAspect As Long
    lIndex As Long
    TYMED As Long
End Type
Private Type DROPFILES
    pFiles As Long
    ptX As Long
    ptY As Long
    fNC As Long
    fWide As Long
End Type
Private Type STGMEDIUM
    TYMED As Long
    Data As Long
    pUnkForRelease As IUnknown
End Type
' ////////////////////////////////////////////////////////////////


' used to create the checkerboard pattern on demand
Private Declare Function FillRect Lib "user32.dll" (ByVal hDC As Long, ByRef lpRect As RECT, ByVal hBrush As Long) As Long
Private Declare Function CreateSolidBrush Lib "gdi32.dll" (ByVal crColor As Long) As Long
Private Declare Function OffsetRect Lib "user32.dll" (ByRef lpRect As RECT, ByVal X As Long, ByVal Y As Long) As Long
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

' used when saving an image or part of the image
Private Declare Function VarPtrArray Lib "msvbvm60.dll" Alias "VarPtr" (Ptr() As Any) As Long
Private Type SafeArray
    cDims As Integer
    fFeatures As Integer
    cbElements As Long
    cLocks As Long
    pvData As Long
    rgSABound(0 To 3) As Long ' reusable UDT for 1 & 2 dim arrays
    ' array items: 0=1D item count, 1=1D LBound, 2=2D item count, 3=2D LBound
End Type

Private Type ICONINFO
    fIcon As Long
    xHotspot As Long
    yHotspot As Long
    hbmMask As Long
    hbmColor As Long
End Type
Private Type BITMAPINFOHEADER
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type
Private Type BITMAPINFO
    bmiHeader As BITMAPINFOHEADER
    bmiPalette As Long
End Type

Private Enum eOScapability
    osAlphaBlendUsable = 1  ' then AlphaBlend enabled & used when needed
    osGDIplusUsable = 2     ' then GDI+ enabled & used when needed (set in isGDIplusEnabled)
   'oszLIBusable = 4        ' then zLib enabled & can be used to create/read PNGs (no longer used, tested as needed)
    osWin2KorBetter = 8     ' AlphaBlend capable system else it isn't for these classes. See isAlphaBlendFriendly property for more info
    osWin98MEonly = 16      ' Win98 or WinME. When m_OScap includes osWin98MEonly+osAlphaBlendUsable then user overrode isAlphaBlendFriendly
    osGDIplusNotAvail = 32  ' then NT4 w/less than SP6 or Win95. Otherwise system is GDI+ capable else it isn't
    osIsNT = 64             ' NT-based system. Unicode compatible
End Enum

Public Enum eImageFormat    ' source image format
    imgError = -1  ' no DIB has been initialized
    imgNone = 0    ' no image loaded
    imgBitmap = 1  ' standard bitmap or jpg
    imgIcon = 3    ' standard icon
    imgWMF = 2     ' windows meta file
    imgEMF = 4     ' enhanced WMF
    imgCursor = 5  ' standard cursor
    imgBmpARGB = 6  ' 32bpp bitmap where RGB is not pre-multiplied
    imgBmpPARGB = 7 ' 32bpp bitmap where RGB is pre-multiplied
    imgIconARGB = 8 ' XP-type icon; 32bpp ARGB
    imgGIF = 9      ' gif; if class.Alpha=True, then transparent GIF
    imgPNG = 10     ' PNG image
    imgPNGicon = 11 ' PNG in icon file (Vista)
    imgCursorARGB = 12 ' alpha blended cursors? do they exist yet?
    imgCheckerBoard = 64 ' image is displaying own checkerboard pattern; no true image
End Enum

Public Enum eConstants      ' See SourceIconSizes
    HIGH_COLOR = &HFFFF00
    TRUE_COLOR = &HFF000000
    TRUE_COLOR_ALPHA = &HFFFFFFFF
End Enum

Private m_Tag As Variant            ' user-defined, user usage only. See TAG property
Private m_StretchQuality As Boolean ' if true will use BiLinear or better interpolation
Private m_Handle As Long        ' handle to 32bpp DIB
Private m_Pointer As Long       ' pointer to DIB bits
Private m_Height As Long        ' height of DIB
Private m_Width As Long         ' width of DIB
Private m_hDC As Long           ' DC if self-managing one
Private m_prevObj As Long       ' object deselected from DC when needed
Private m_osCAP As eOScapability ' See eOScapability enumeration above
Private m_Format As eImageFormat ' type of source image
Private m_ManageDC As Boolean   ' does class manage its own DC
Private m_AlphaImage As Boolean ' does the DIB contain alpha/transparency
Private m_GDItoken As Long
Private m_ImageByteCache() As Byte  ' should you want the DIB class to cache original bytes
' ^^ N/A if image is loaded by handle, stdPicture, or resource

Private m_GDIplus As cGDIPlus   ' maintains instance to speed up drawing
Private m_KeepGDIplusActive As Boolean

' NEW SECTION *******************************************************************************
'                   CLASS INITIALIZATION & TERMINATION ROUTINES
' *******************************************************************************************


Private Sub Class_Initialize()

    ' Determine operating system for compatibility of 32bpp images
    ' http://vbnet.mvps.org/code/helpers/iswinversion.htm
    ' http://msdn2.microsoft.com/en-gb/library/ms724834.aspx
    
   Dim osType As OSVERSIONINFOEX
   Const VER_PLATFORM_WIN32_WINDOWS As Long = 1

   ' Retrieve version data for OS.
   osType.dwOSVersionInfoSize = Len(osType)
   If GetVersionEx(osType) = 0 Then
      ' The OSVERSIONINFOEX structure is only supported
      ' in NT4/SP6+ and NT5.x, so we're likely running
      ' on an earlier version of Windows. Revert structure
      ' size to OSVERSIONINFO and try again.
      osType.dwOSVersionInfoSize = Len(osType) - 8
      Call GetVersionEx(osType)
   End If
   
    If osType.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS Then

        If osType.dwMinorVersion = 0 Then ' Win95; can't use AlphaBlend nor GDI+
            m_osCAP = osGDIplusNotAvail

        Else ' flag as Alphablend disabled, but capable & is Win98/ME
            m_osCAP = osWin98MEonly
        End If

    Else

        If osType.dwMajorVersion > 4 Then ' if Win2K or better
            m_osCAP = osAlphaBlendUsable Or osWin2KorBetter ' flag as AlphaBlend enabled (Win2K or better) and capable

        Else ' WinNT4. If SP6 or better than GDI+ capable else not. Regardless, not AlphaBlend capable
            If osType.wServicePackMajor < 6 Then m_osCAP = osGDIplusNotAvail
        End If
        m_osCAP = m_osCAP Or osIsNT

    End If

    Me.isGDIplusEnabled = True  ' attempt to start GDI+, test system capability, add osGDIplusUsable to m_OScap
    If Me.isGDIplusEnabled Then Me.HighQualityInterpolation = True
    
    m_Format = imgError

End Sub

Private Sub Class_Terminate()
    DestroyDIB ' simply clean up
End Sub

' NEW SECTION *******************************************************************************
'                           PUBLIC PROPERTIES AND METHODS
' *******************************************************************************************


Public Property Let Alpha(isAlpha As Boolean)
    m_AlphaImage = isAlpha  ' determines the flags used for AlphaBlend API
    ' this flag is set by the various image parsers; setting it yourself
    ' can produce less than desirable effects.
    ' Used in Me.Render & Me.TrimImage, cPNGwriter.OptimizeTrueColor & cPNGwriter.PalettizeImage
End Property
Public Property Get Alpha() As Boolean
    Alpha = m_AlphaImage
End Property

Public Property Let HighQualityInterpolation(Value As Boolean)
    ' When possible GDI+ will be used for stretching & rotation.
    ' If GDI+ is used,then high quality equates to BiCubic interpolation
    ' If not used, then BiLinear (manual processing) will be used.
    ' If High Quality is false, then Nearest Neighbor (very fast, less quality) interpolation used
    m_StretchQuality = Value
End Property
Public Property Get HighQualityInterpolation() As Boolean
    HighQualityInterpolation = m_StretchQuality
End Property

Public Property Get ImageType() As eImageFormat
    ImageType = m_Format    ' returns image format of the source image
End Property
Friend Property Let ImageType(iType As eImageFormat)
    m_Format = iType    ' set by the various image parsers. This is not used
    ' anywhere in these classes, you can do with it what you want -- for now.
End Property

Public Property Let ManageOwnDC(bManage As Boolean)
    ' Determines whether or not this class will manage its own DC
    ' If false, then a DC is created each time the image needs to be Rendered
    Dim tDC As Long
    If bManage = False Then     ' removing management of DC
        If Not m_hDC = 0& Then   ' DC does exist, destroy it
            ' first remove the dib, if one exists
            If Not m_Handle = 0& Then SelectObject m_hDC, m_prevObj
            m_prevObj = 0&
        End If
        DeleteDC m_hDC
        m_hDC = 0&
    Else                        ' allowing creation of dc
        If m_hDC = 0& Then        ' create DC only if we have a dib to put in it
            If Not m_Handle = 0& Then
                tDC = GetDC(0&)
                m_hDC = CreateCompatibleDC(tDC)
                ReleaseDC 0&, tDC
            End If
        End If
    End If
    m_ManageDC = bManage
End Property
Public Property Get ManageOwnDC() As Boolean
    ManageOwnDC = m_ManageDC
End Property

Public Property Get isAlphaBlendFriendly() As Boolean
    isAlphaBlendFriendly = ((m_osCAP And osAlphaBlendUsable) = osAlphaBlendUsable)
    ' WinNT4 & below and Win95 are not shipped with msimg32.dll (AlphaBlend API)
    ' Win98 has bugs & would believe that WinME is buggy too but don't know for sure
    ' Therefore, the Rendering in this class will not use AlphaBlend on these
    ' operating systems even if the DLL exists, but will use GDI+ if available
    ' Can be overridden by setting this property to True
End Property
Public Property Let isAlphaBlendFriendly(Enabled As Boolean)
    ' This has been provided to override safety of using AlphaBlend on Win9x systems.
    ' Caution. Only set this when rendering to a known device dependent bitmap (DDB)
    ' Alphablend can crash when rendering DIB to DIB vs DIB to DDB. Be warned.
    If Enabled = True Then
        ' Overriding in play: allow AlphaBlend if system is Win98 or better
        ' By default this is already set for Win2K or better
        If ((m_osCAP And osWin2KorBetter) = osWin2KorBetter) Then
            m_osCAP = m_osCAP Or osAlphaBlendUsable
        ElseIf ((m_osCAP And osWin98MEonly) = osWin98MEonly) Then
            m_osCAP = m_osCAP Or osAlphaBlendUsable
        End If
    Else
        m_osCAP = m_osCAP And Not osAlphaBlendUsable ' disallow AlphaBlend
    End If
End Property
Public Property Get isGDIplusEnabled() As Boolean
    ' identifies if GDI+ is usable on the system.
    ' Before this property is set, GDI+ is tested to ensure it is usable
    isGDIplusEnabled = ((m_osCAP And osGDIplusUsable) = osGDIplusUsable)
End Property
Public Property Let isGDIplusEnabled(Enabled As Boolean)
    ' Sets the property. If set to False by you, GDI+ will not be used
    ' for any rendering, but still may be used to create PNG files if needed
    
    ' You can reset it to true at any time. If the system won't support
    ' GDI+, then the True setting will simply be ignored -- no harm, no foul
    ' To test success:  c32class.isGDIplusEnabled=True: If c32class.isGDIplusEnabled=True Then ' success
    
    ' Note. My GDI+ dll works on Win95. Therefore, the O/S capabilities check below
    ' has been removed to create a hard test vs simply assuming
    
    If Not Enabled = Me.isGDIplusEnabled Then
        m_osCAP = (m_osCAP And Not osGDIplusUsable)
        If Enabled Then
'            If (m_osCAP And osGDIplusNotAvail) = 0 Then ' else Win95, NT4 SP5 or lower
                Dim cGDIp As cGDIPlus
                If m_GDIplus Is Nothing Then
                    Set cGDIp = New cGDIPlus
                Else
                    Set cGDIp = m_GDIplus
                End If
                If cGDIp.isGDIplusOk() = True Then m_osCAP = m_osCAP Or osGDIplusUsable
'            End If
        End If
    End If
End Property

Public Property Let gdiToken(Token As Long)
    ' Everytime a GDI+ API function is called, the class calls GDI+ apis to
    ' create a GDI+ token first then destroys the token after the function is called.
    
    ' This occurs quite often. However, you can create your own token by calling
    ' GdiplusStartup and then passing the token to each class for the class to use.
    ' You would call GdiplusShutdown during your main form's Terminate event to
    ' release the Token.
    
    ' When Token is zero, the classes will revert to creating a token on demand.
    ' When the Token is not zero, any other DIB class created by this class will
    ' pass the token as needed. The only routine that can create a new instance and
    ' returns that new instance is the CreateDropShadow method. You must set the
    ' token for that class at some point for that dropshadow class to use it.
    
    m_GDItoken = Token
    
End Property
Public Property Get gdiToken() As Long
    ' returns the GDI+ token if one was created
    gdiToken = m_GDItoken
End Property

Public Property Let Tag(vValue As Variant)
    If IsObject(vValue) Then
        Set m_Tag = vValue
    Else
        m_Tag = vValue
    End If
End Property
Public Property Set Tag(vValue As Variant)
    Me.Tag = vValue ' use the object check in the Let property
End Property
Public Property Get Tag() As Variant
    If IsObject(m_Tag) Then
        Set Tag = m_Tag
    Else
        Tag = m_Tag
    End If
End Property


' NEW SECTION *******************************************************************************
'                               PUBLIC READ-ONLY PROPERTIES
' *******************************************************************************************

Public Property Get Width() As Long
    Width = m_Width     ' width of image in pixels
End Property
Public Property Get Height() As Long
    Height = m_Height   ' height of image in pixels
End Property
Public Property Get BitsPointer() As Long
    BitsPointer = m_Pointer ' pointer to the bits of the image
End Property
Public Property Get scanWidth() As Long
    scanWidth = m_Width * 4&    ' number of bytes per scan line
End Property
Public Property Get Handle() As Long
    Handle = m_Handle   ' the picture handle of the image
End Property
Public Property Get isZlibEnabled() As Boolean
    ' Read Only
    ' To create PNG files, GDI+ or zLib is required. This property informs
    ' you if zLIB exists in the system's DLL path
    isZlibEnabled = iparseValidateZLIB(vbNullString, 0, False, False, True)
    
End Property

' This setting will keep the current cGDI+ class active which
' prevents destroying the GDI+ hImage and Token objects.  This
' can speed up rendering. This option SHOULD NOT be set when the
' class is not compiled. Else crashes will happen when
' user hits END or VB toolbar's STOP button while in IDE
Public Property Get KeepGDIplusActive() As Boolean
    KeepGDIplusActive = m_KeepGDIplusActive
End Property
Public Property Let KeepGDIplusActive(keepActive As Boolean)
    m_KeepGDIplusActive = keepActive
    If keepActive = False Then
        Set m_GDIplus = Nothing
    ElseIf Me.isGDIplusEnabled = False Then
        ' can't set to True if GDI+ not installed
        m_KeepGDIplusActive = False
    End If
End Property


' NEW SECTION *******************************************************************************
'                                       PUBLIC METHODS
' *******************************************************************************************

Public Function LoadPicture_File(ByVal FileName As String, _
                                Optional ByVal SaveFormat As Boolean) As Boolean

    ' PURPOSE: Convert passed image file into a 32bpp image
    
    ' Parameters.
    ' FileName :: full path of file. Validation occurs before we continue
    ' SaveFormat :: if true, then the image will be cached as a byte array only
    '   if the image was successfully loaded. Call GetOrginalFormat to retrieve them.
    
    ' Why would you want to save the bytes? If this is being used in a usercontrol,
    ' saving the bytes will almost always be less size than saving the 32bit DIB.
    ' Additionally, these classes have the ability to get different sizes from
    ' the original source (i.e., WMF, icon, cursors) if available, but if the
    ' 32bit DIB is saved, it is a constant size. The potential of different sizes
    ' could allow better resizing of the image vs stretching the DIB.

    On Error Resume Next
    Dim hFile As Long
    
    hFile = iparseGetFileHandle(FileName, True, ((m_osCAP And osIsNT) = osIsNT))
    If hFile = INVALID_HANDLE_VALUE Then Exit Function
    
    If GetFileSize(hFile, 0&) > 56 Then
        
        ' no image file/stream can be less than 57 bytes and still be an image
        Dim aDIB() As Byte  ' dummy array
        LoadPicture_File = spt_LoadPictureEx(hFile, FileName, aDIB(), 0&, 0&, SaveFormat)
    
    End If
    CloseHandle hFile
    
End Function

Public Function LoadPicture_Stream(inStream() As Byte, _
                                    Optional ByVal streamStart As Long = 0&, _
                                    Optional ByVal streamLength As Long = 0&, _
                                    Optional ByVal SaveFormat As Boolean) As Boolean
    
    ' PURPOSE: Convert passed array into a 32bpp image
    
    ' Parameters.
    ' inStream:: byte stream containing the image. Validation occurs below
    ' streamStart :: array position of 1st byte of the image file. Validated.
    ' streamLength :: total length of the image file. Validated.
    ' SaveFormat :: if true, then the image will be cached as a byte array only
    '   if the image was successfully loaded. Call GetOrginalFormat to retrieve them.
    
    ' Why would you want to save the bytes? If this is being used in a usercontrol,
    ' saving the bytes will almost always be less size than saving the 32bit DIB.
    ' Additionally, these classes have the ability to get different sizes from
    ' the original source (i.e., WMF, icon, cursors) if available, but if the
    ' 32bit DIB is saved, it is a constant size. The potential of different sizes
    ' could allow better resizing of the image vs stretching the DIB.
    
    If iparseIsArrayEmpty(VarPtrArray(inStream)) = 0& Then Exit Function
    If streamStart < LBound(inStream) Then streamStart = LBound(inStream)
    If streamLength = 0& Then streamLength = UBound(inStream) - streamStart + 1&
    If streamLength < 57 Then Exit Function
    ' no image file/stream can be less than 57 bytes and still be an image
    LoadPicture_Stream = spt_LoadPictureEx(0&, vbNullString, inStream, streamStart, streamLength, SaveFormat)

End Function

Public Function LoadPicture_Resource(ByVal ResIndex As Variant, ByVal ResSection As Variant, _
                            Optional VBglobal As IUnknown, _
                            Optional ByVal streamStart As Long = 0&, _
                            Optional ByVal streamLength As Long = 0&, _
                            Optional ByVal SaveFormat As Boolean) As Boolean

    ' PURPOSE: Convert passed resource into a 32bpp image
    
    ' Parameters.
    ' ResIndex :: the resource file index (i.e., 101)
    ' ResSection :: one of the VB LoadResConstants or String value of
    '       your resource section, i.e., vbResBitmap, vbResIcon, "Custom", etc
    ' VbGlobal :: pass as VB.GLOBAL of the project containing the resource file
    '       - Allows class to be mobile; can exist in DLL or OCX
    '       - if not provided, class will use resource from existing workspace
    '       - For example, if this class was in a compiled OCX, then the only way
    '           to use the host's resource file is passing the host's VB.Global reference
    ' streamStart :: array position of 1st byte of the image file. Validated.
    ' streamLength :: total length of the image file. Validated.
    '   -- See LoadPicture_Stream for the validation
    
    ' Tips:
    ' 1) Store 32bpp bitmaps in the "Custom" resource always. Storing in the
    '       Bitmap resource can change color depth of the image created by VB
    '       depending on your screen settings
    ' 2) Icons, normal bitmaps, & cursors are generally stored in their own sections
    '       However, with icons containing multiple formats, VB will extract the
    '       closest format to 32x32. May want to consider storing these in "Custom"
    ' 3) All other types of images are normally stored in the "Custom" section

    On Error GoTo ExitRoutine
    
    Dim oWorkSpace As VB.Global, tPic As StdPicture
    
    If VBglobal Is Nothing Then
        Set oWorkSpace = VB.Global
    ElseIf TypeOf VBglobal Is VB.Global Then
        Set oWorkSpace = VBglobal
    Else
        Set oWorkSpace = VB.Global
    End If
    
    If VarType(ResSection) = vbString Then
        Dim inStream() As Byte
        ' could be anything, PNG,icon,gif,32bpp bitmap,wmf, etc
        inStream = oWorkSpace.LoadResData(ResIndex, ResSection)
        LoadPicture_Resource = LoadPicture_Stream(inStream, streamStart, streamLength, SaveFormat)
    End If
    LoadPicture_Resource = Not (m_Handle = 0&)
    
ExitRoutine:
    If Err Then Err.Clear
End Function

Public Function LoadPicture_FromOrignalFormat() As Boolean

    ' PURPOSE: Reload the current image from the cached bytes (if any)
    ' If the original bytes were not cached when the image was loaded, then no action
    ' will be taken.  See LoadPicture_File & LoadPicture_Stream
    
    Dim tBytes() As Byte
    tBytes() = m_ImageByteCache() ' copy bytes; original are destroyed when DIB is recreated
    LoadPicture_FromOrignalFormat = Me.LoadPicture_Stream(tBytes, , , True)
    
End Function

Public Function GetPixel(ByVal X As Long, ByVal Y As Long, Optional ByRef AlphaValue As Long, _
                            Optional ByRef asPreMultiplied As Boolean) As Long

    ' Function will return the pixel color value and alpha value from the DIB
    ' Note that the DIB is always referenced top down within this function
    
    ' X is the left coordinate of the pixel to be returned, image always starts at 0,0
    ' Y is the top coordinate of the pixel to be returned, image always starts at 0,0
    ' AlphaValue will contain the alpha value of the pixel.
    ' asPreMultiplied. If false, then premultiplication is removed else it isn't
    
    ' Return value is the RGB color value of the pixel.
    ' If return value is -1 then the X,Y coordinates passed are invalid
    
    ' It is far more efficient to use GetDIBits or overlaying your own array when more than
    ' one pixel is required to be returned; in other words, recommend not using this
    ' function within a loop
    
    AlphaValue = 0&
    If X < 0& Or X > m_Width - 1& Then
        GetPixel = -1&
    ElseIf Y < 0& Or Y > m_Height - 1& Then
        GetPixel = -1&
    Else
        Dim pOffset As Long, pColor As Long
        ' calculate the location of the X,Y coordinate in relation to a bottom-up DIB
        pOffset = iparseSafeOffset(m_Pointer, X * 4& + ((m_Height - Y - 1&) * m_Width * 4&))
        
        ' get the alpha value
        CopyMemory AlphaValue, ByVal iparseSafeOffset(pOffset, 3&), 1&
        
        ' get the pixel color & convert it to RGB
        CopyMemory pColor, ByVal pOffset, 3&
        If asPreMultiplied = True Or (AlphaValue Mod 255) = 0 Then
            GetPixel = ((pColor And &HFF) * &H10000) Or ((pColor \ &H100) And &HFF) * &H100 Or ((pColor \ &H10000) And &HFF)
        Else    ' remove premultiplication
            pOffset = ((255&tedoy
   ri
    idth As on the system.
    ' Before thisoal pOffsi.e.,  PUBLI   ' Ber <d &H?th As on) *Xe thi
'    XamLength < <d &H?th AsT rg nd stset(m_Po

' // ublic Property Ge &H?th A) Is VB.***T&*********jnStream)) = 0&epGDIplusActe 
    ' Beforc ProplROPERTIES Aalways rei*,lusOk() = Trueoperty (he pixel.
    ' If  As on) *Xe thi
'    XamLength < <d &H?th Asd test vs sim          Optional ByVal SaveFormat As Boolean) As Boole  Optiwty
Private m_StretchQuality  ((pColor \ &H10al ByVal SaveFwg, pColor As Long
        ' con
    imgsettings
    'rivate Doy
   rolor \ & f the original bytes were not cached when the image was loaded, sWaS=T&*******Cave the bytes? If th to overrRl
    ' Non Lodcroperty


Ns rei*,lusOk() =nate in rel3ream)) = 0&epGDI
    nything,Sws class to be #set = iparseSafeOffset(m_Pointer, X  areE&: array p****Cave t
    ' To  ' NonaCAP And osIs1O>Publar, X  oadPicture_ oeei*,lusOivert to creating a token on demand.
    ' When the Token is nel3ream)s osWin98MEonly+osAlphaBlendUsable then user overrode isAlphaBlendFriendly
   le then user overrode e.,lusOiveradPi(ifDoy
   rolor \ 'izing of the image vmnDC(bManage As Bse the bytes'izOffset, es'izOffset,lor \ &H100) And &HFF) * &N to be #set = g of the image *reE&rrod.      (fB+E&rrod.   sPreMultiplied As Boolean) As LonVarPt X * 4&m)am eATETC
    cfFormat Multiplied As Boolean) As Lonen th(v     (fB+E&rrod.)e isAlphaBlenCer v When tham e    (See Loaa If (meginalFormatp tes? If pixel.
    ' dPicture_E the alpha Routineatp tes? I dPicture_swdwaEarray only
    ' nel3ream)s osWin98MEonly+osAlphaBlends pre*******

Public Property Get Width() A=******** )y+osneatp'0idth of ima3TETC
B crenly+os GetVersionEx(osType) = 0 oved else it isn4
Public ProLon.a Set cGDIp = m_Trueoperty (he pimat         \ &
    cse    pPa As Long
Privat

        If osType.dwMinorVersi
Public  nythtream:: bytwMinorropert sWaS=T&*******Cave the bytes? If th to ovei*,lusOk() =nate in rel3ream)) = 0&epGDI
    nything,Sws class to be #set t -e -1 then the X,Y coordinates passed are invmend noto be irv When tha *Xe thi
'    Xaource = Not (m_H+ ((m_Hes If th to ovei*,lusOk() =nate-rv When tha *Xe thi
'    Xaource = Not (m_H+ ((m_Hes If th to    ' could ak BooIAgnd &HFF) -e = No Not (ms osWin98MEon -e = No rt + 1&
    If stre.hen zLib enabled & can be used to cii    ' Non Lodcroperle Loaa If (mY > m_Heige(FileName, True, ((m_ the various image parsers; setting it yom)) = 0&epG\ &H10'4sed sype)
   IseSafeOepG\ &H10'4sed h() A=*** ed h() A=*** ed h() A=* As Lo'&H10'4sed sy ed h(H10'4sed h() A=*** ed h() A=**o'&Ho 4sed sc Function Lo Then        ' creatd are invmend noto ation(Va,in2K or better
       4sed sc Funct used w not(Va,in2i////****d sc Function Le)
   IseSafeOepG\ &H10'4sed h() A=*** ed h() A=*** ed h(epG\ &H10=not use Alp As Boolean 'ton
aIH10'4sed h(((((((0  If Me.isGD      ' user-defineAs on) *Xe thi
'    XmINVALIerrode          pOffset = ((255&tedoy
   ri
    idth As on the system.
    ' Befor AlphaBlend=bm)) = 
   will be cachpOffset = ((Width() ABlend=bm))ure_swdwaEnroperle Lond=bm))urmbm))ure_sm))urean 'ton
aIH10' As Boole &H1As Boolean 'ton
aIH10'4sed h(((((((0  If Me.isGD      ' useaS If tt(Va,in2i//Offse8/MEle &H1As t(Va,in2i//OfRer
 V    4sed**jnStream)) = 0   ' Retueng management of DC
        If Not m_hDC = 0& Then   ' DC a *Xe tttttIf keepActive = Fng
End APreMult:ity  ((pColor \ &H10al ByVal SaveFwg'h    ng used inaDeclare Functifnxel to be returneMolean 'tonuser-defsRever)class to cache orsoeMoleaI+ wilanagent of DCleaI+ wilanagent of DCleaI+ wilanagent of DCleaI+ wilanagent of DCleaI+ wilanages than 57 bty Let aI+ wilpe
Privage
:GDIplxrivage
:GDIpn 'ton
aIH10' AaI+ wiSage
:GDIplxrivage
:GDIpn 'ton
aIage

Privage
nmltipliAMe Loaa Ifon
aIage

osCAP As eOS(R////////n2i//OfRer
  If Mu IseSafeOep n 57 bty Let a<al SaveFwg'h    n olAaI+ wiSvurce file index (i.e., 101)
    ' ResSect to be returt capT ResSen o10' 
   aed h((01)
    ' ResSect tl a ResSee5
    
    ' X isf Not Bof DCleu IseSafeOep n 5 Le)
   IseSafeOepG\ o;l a  BitsPointer = m_P IseSafeOepG\ &H10'4sed h() A=*** ed h() A=*** ed h() A=* IseSafeOepG\ o;l a  BitsPointer = m_P IseSaeC a  B(m_osean 'tonprevObjrignalFormat() perty
Public PrBooleaT,d h() A=*** ed h() A=*** ed h() A=* IseSafeOepG\ onEx(osType) = 0 oved els 0&, tDC
            End If
nt of DCleaI+ YEnd Ifhttp://vbnet.mvps.org/code/hemagea,,y5 hFile
    
E DC
        If Not mwill be tak.s eOS(R/////) to be retureOffseffot ore this property is set, GDI+ is tested to ens     ' size e always s   rolor \ & f the orif NA=*Space AslAaI+ wiSvurce file 0'4oleaI+ic PrBoolwFXe ttttways s   rolor \ & f the orif NA=*w 0'4oleaI+ie oriend disablrifp:
    If Err Then Err.Clear
Endidt_osean 'tonpA    ' The OSThen Err.Clear
Endidt_osean 'tonpA    ' The OSThen Err.Clear
Endidt_os
En
Endidtr ' flag as AlphasGD      ' u image created by VB
 tr ' flag  using th ah3roplROPE eu Ii  DimoDCl:sing th ah3roplR*he imaglsosean 'tonpdtr ' fed scGLOBAL of the project F    'Value, ByVal iAP Or ospn:1en   ' Dge

Prily clean up
End Sub
iAP p****Cave t
    'BitsPointer = m_P func_oimaglsohaBlend=bm)) = 
   will be cachpO.mvps.haBlendglsohaBlend=bm))  tDC
O,(he po a bottom-up DIB
     cii    ay only
    ' neo use thi) Then
   Type As eImageFoadPicture_ oeei*,lu           Prytime a GDI+ API fuh ah3roplROPE eu Ii agement of Daglsoh() A=*** edXhen Err.Cln up
End .streamL Etos
En
En)urean 'ton
ut to True if GDI+ not installed
        h ah3r ' BeI=W)on
   PropertyE overWdddddddd(+ APf GDI+        Fddddddd(+E overWd,eo_osean 'tonprevObjrignalFormat() perty
Public        Fddddddd(+rev.iend disablrifp:
    If Err Then Err.Clear
Enth ah3roplROPE eu Ii  DimoDCl:sing th ah3roplR*he imaglsosean 'tonpdtr ' fed scg'h  Ind=bm))  tDlier vers.
'h  Ind=bm))  tDlier vers.
'h  Ind=bm))  tDlier vers.
'h  Ind=bm))  tDlier vers.
'h  Ind=bm))  tDlier vers.
'h  Ind=bm))  tDity = VVVVVVVVVVVVVVVVVs Boolean
 vate m_o&)
          rdinateswkn Err.Clnot pro) A=*** ed h() A=hkeepActpdtr 'c Function LosGD   oDClbm)) ((Width() ABlend=bq'A=hke        Set m_GDI)) ((s  oDClbm)) ((Widt   rolSe invmend nng ma)usActive() As Boolean
    KeePNG = 10     ' Pwkn Err.Clnot pro) A=*** ed hdddi ah3roplROPE eu Ii  DimoDCl:sing th ah3roplR*he ima)usActiv. 4sed sc various image parsers; I&.!p' coulderally stored in their own sections
    '       However, with icons containing multiple formats, VB will cse    pPa As Lo- stor,Len Euce less than deble onsg Lo- stier ***Che orif  pPa r ***Chly+osAlphaBlendUsaoe destroyeFY DClae leplR*he ima)usActiv. 4sed ssActiv. 4sed sc variouf= Me.LoadP    Dim 
   Type As eImageFoadPictu_P IseSaeC a  B
   Tee TC a  B
   Tee TC a  B
   ( storrmats, VGDIplu The of0meH100n = TokeD=ion >eD=ion >eD=ion >eD=ion >eD=iof:sing thi Overriource(B
Endidof0meHf  B
   yes
   b        pOffset = b        pOffset = b       Releasif NAreaurcen >eD=iod Sub
ihe ima)ueD=iod Subs eImageFoadPictudof0mendent bsin   Tto ens     ' size eC a ub
ihe ima)ueD=iod Subs e*************************ph3roplROPE eu I only
    ' nelcr vers.
'h VVVVVVVs Booleanh() A=*** ed h(epChly+osAlnything,Sws class th    ' for any renbong, pCoH(h() A=*** iTel = -1&Lble
'nd still be an ipe
PrivaeD=iod Siod  want -- for n the bytes'izOffset, es'izOffset,lor \ &H100) And &HFF) * &N to be #set'izOff, es'izof= Me.Loas & woule is the CreateDropdof0menn, i.e., ipe
PrivaeD=iod Siod  wanizOff, es'izof= Mthe CreateDro = vbStr.Loasv
Enth lobal =Ie is the CreateDropdomINVALIerrlaying eDro = voate Doy
 o create PNG fibStr.Loasv
Enth lobal'       es'izof= Me.Loas & woule is tvalue  is ts rewoule is ImageFoadPictudof0mendent bsin   TtBaes'izof= check below
    E eu I only
    Ybe s'izof= Me.Loes'iz for n the bytesC   ' pass the token asdYal bitmaps, & cdtr ' fed scg'h  Ind=bDC =Ma= Me.Loar****************PNG in.And &HFF) * &' could ak BoSthe CreatiTel = -oonpdtr ' fed    Relo************PNG in.A for n eFoadPicture_ oeei*,lel =  en Errtion)
        LoatiTel = RparseSa^n)
    /i*,lel =  en Errtion)
        LoatiTel = RparseSa^n)
    /i*,lel =  een Errtion)
)ubled=True Then&Cd=True Then&Cd=True various imagedtr ' fen&Cd=True Theeeeeeeeeeeeeh() A=**o'&Ho 4seseSa^n)
 -1&L     eooperty (h^n)
 -et = ((Wideeeeeeh() A=**rus imagedtr ' fNvarious imagedtr ' fen&Cd=True Theeeeeeeeeeeeeh() A=**o'&Ho 4seseSa^n)
 -1&L    isGD(hen
u I on>erty Get ManageOwnDC() As Boolean
    ManageOwn within this function
    
    ' X ma)ueD=i= VVf7Cd=True Theeeeens0own be motr ' fenreeens0ownr Err.Ctr ' feed hdsIfaTrue Th0own brt Manabns0ownr Err.Ctr ' feRPOS ThmseSafeO -1&L Type As eImve the ability to  Mu IscUEhe abilit e formats, VB will csobilit e0own be L    isGctusMs'izof= Cd=Trunter to &dsIfaTrue Th0own brt Manabns0ownr Err.Ctr ' feRPOS ThmseSafeO -1&L Type As e Err.Ctr ' if the image wan eFoadPictur'2 vbStr0own be motr lass will uafeOepG\ &H10'4sed h() AUtepG\ &H\ o;l a  BitsPointer = m_P IseSafeOepG\ &H+'nh(((((((0 ilass will uafeter =Alphadoes exisc Err.Ctr ' if the im-=sdoes exist, destroy i'l:sing theSafeO -1&L uafeteeeeeeeeeeeeeeeeeeeeeeI+ wiSvuType As eImve the abiart, streamLength, S    XmINVALIerrode          pOffsdTuTypt&L Type As eImmmmeeeeei bility t ' returns the GDI+ tooooooooooooooooooooooooooe t
    z + wiS i.    ' number of bytes per sc XmINVALIerrode  he GDI+ tokehe imagms.
s&dsIfty
End o- st**********G 4s +deXing, PNGehe imagms.
Dity =alse The GDI+eter =Alp Set cGDcbits of thh() AUtepG\*******Gehe imaguafetlts of t)OepG\ 2 by VB
   g theSafeO - ' feRRRRRRRRRRRRRRRRRRRRRial SaveFwg'h    n olAar main oet cGof DCleae*Chly+osAlphaBlendUsaoe destroyeFY DClae leplR*he ima)usActiv.wsted c*1oyeFY DO>fwd=Trunter to &dsIfaTrue Th0own brt ManSn
'   Bnbong, pns the eateDropdomINVALIerrlaying eDro = voate Do,eo_oseates? If thi( &H10ot Bof.e thi
'    XamLengns th thi( &H10otrmats, VB wil
    
    'pe
PrivaeD=iod Sm maiates? e Do,eo &H10otrmats, es? Iflc0otrmats, es? I
Pubc0otrpeo t e formats, VB 'lendUsa mai=pD() AU pertuse eamStart :: array position       S KmseSar main pertuse eam Manabuse '2 vbStr0own ay po(nter to &dsIfMEle &H1hmseSafe)) ((Widt   rolSe the pixel.
RRRRRRR\ 2 by VB
   g(es e6 tDliepmray posiet,lor \OffseH+'phaBlendUsable ' disallow Alpha to be r formats, VB in.And &HFF) * s1O>Publar, X  oadPicture_ oeei*,luhe im-=sdh() Aoosted c*1oyeFY DO>obal As IUp ats, VB in.And &HFF) * s1() Aoosted 
Enth + wilanagent of DCleaI+ wilanagent of DCleaI+ wilanageof DCl4C#Alphadoes exisc Err.Ctr '   XamLengns th tsd Property
Public Ibsin   Tto ens     ' size eC a ubats, VB in.And &en Err.Clear
Enns teS in.Anistedmend nng m As Ifwd=Trunter to Ii  DimoDCloyeFY DO>obal I+ API fuh ah3rop, X  loyeFY DO>obal I+ API fuh ah3rop, X  loyeFY DO>obaaaa'   XaO>obaaaa'   XaO>op, XLengnve t
    'BifaTrue h     d If
s API fuertuse eam MaAPI fuertuGDI
  fuh ah3rop, X  loyeFY yeFYuertuGDIU.And CAP And osGDIplusNotAvail) = 0 Then ' else Win95, NT4 SP5 or lower
       X&dsIfwd CAP Arily cIerrla' Beforc ProplROPERTIES Aalways rei*,lusOk() = Trueoperty (he pixr Alpha
    
vs AalwrrrrrusOk() = TrueopertGDIplusNddddddddd othees? Iflc0otrmats, es?he pixr.  yed IflcnhaBlendUlean 'tonuser-defolwrrrrrusOk(eFoadPictur'2 vb
vsor.  yed I+ or zLib is requ_Strk+'r or zLisPicl uafete   CopyMeidth() ABlend=bt a<alotrmats, VB wil
 essssssssssSP5 .FoadPictmI on> DimoDCl:sing tclusOk() = Trueoperty vn Err.ClearOk() = Trueoperty vn ng of tng tclusOk()tar else Winmve the ability to  MuCAP And osGDIplusNotAvail) = 0 Then ' elmI on>oyeFY DO>obal I+ API fevert to c osGDIty (he pixr Alp     trmats, VB wPublic IbsiRo& woule iPwkn Err.Clnot pro) A the   Xa proay eeeI+ wiSvuaSon>oyeFY DO>wkn Err.' Err.Ctr ' feRPOS won't supy to            ream)) = 0&rrrrrusme ' deoule i,eOepG\ onEx(osTypssss.Cle'tonpdtr ' fed seoDfaTrue h  (),= 0&rrrr'ton
aIH10osWin98MEonly+osAdhe() ' copy bytypssss.Cle'tonpdtr ' f. -et = (in98MEl:sing tt the'ton
tonpdtr ' f. -et + not installed
        h ah3r ' BeI=W)on
 fXaBlendFriends deoule i,eOepG\ onEx(osTypssss.Cle'tonpdt
 &T& woule iPPPPPPPPPPPfendFriends deoulwiSEdt
 &T& wouyustom" section

  e i,eOepus ddeoulwiSEdt
     h ah3r ' BeI=W)o Creat ' f. -et + noto2ction

t.o'2 vbStsssso &H
 fXaBl           ' first re eC a ubeCFileName A'  =C a utr.Loasv
he projto  Mu IscUEhe abist re eC a ubeCFileName A'  =C a utr.Loasv
he projt,
    ' has been remo less than 57 bytes and still' first D' has beCA      h s beCA      h s bationhe *************************D  h s beeferppdomINVes bh s bationhe *********************sv
he projt,
    ' haes and still' ule i,eOepG\ onEx(  h s beee imaggxerrRl
iPPPPPPPfendF&      LoatiTel = RparseSa^ Rl
iP*****0on SHOULD NOT be set when the
ValidGDIplusA the
do utr.Loasv
he proean

wouyustom" sectihe proean

wouyustom" sectihe proean

llpha
  s eImve thNes and scf= cheY You riginal are d DO>obal I+ APe Do) As Long
    Handle = m_Handle   ' the picture handle of the image
End Paofer, ylseIf TypeOf VBooleaT,d h() *************ure ha set when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t when thDIplu The 
  t wheThe 
  t whhe 
       to  bbbbbbbbbbbbb" section Lon
  t wheThe 
  t whhe  DimoDCl:sing t, es'iz to  bbbplied n thDIplu The 
  t i-
 &T& woule iPctihe proeoee 
  t waalled
ss.+ API fuh ah3 
  t when thDIplu The 
  t when thDIplu The 
(y

I    ' .when thDIplu The 
 bbbpliImve thNes o 
vs Aalwrr
    ' V p (),= 0&rrr, ylseI 
  t ile. Validat   '   Optiosf Err-ke  was in a co0own be L  whhsysteseSaeC a The 
  t when thDI        thDIplu Thehen nter   t  he pon nter= m_P IseSthDIpluim inStre
        Dim inStB will extibStrl (),DLL or OC 
  t
    '       Bitmap resource can changight of ima>eD=iet m_GDIpllllllt whyustom" pllllllt whyo)a       Bitmap resource       BY yeFYuertaided to opiled. ElsImotr ' fenreing tclusOk() 2Et ' f. -et thDIplu Tion
                                 Iplu Th   '  hen thDIplu The 
plu Tion
     ng man
    pG\ on AU t i-
sing t, esd stilaB_
     ng man
    pG\m_GD0&rrr, ylseI 
  talidat   '   Optiosf/e L  w thDIplu The 
plu Tion
     ng man
    pG\ ohen thDIplu The 
he'ton
tfer, Err.Clear
Endage file. Va ,tfer, Err.Clear

    pG\ ohen thDIpl2esd stilaB_
     ng man
  imoDCl:singlu Tg manuooleanm
    
    Dim tBylAaI+ wN se Winmve th file. plu The 
(y

I    ' .when thDIplu e il) = 0 Then ' else Win95,& f te il) an 57 bytes and still be an ims_0 Then ' else Win95,oulwiSEdted. ElsImotr ' fenreinc ke t whe ng of tng tcmStargeFoadPilThen ' else Win95,oulwiSEdted. ElsImotr ' fenrpdof0mefnxel to be ruofenretutr.Loasv' f. -et thDIplu Tion
                   in.A owing createsourcg m As he moDCl:she current image from the cached he current image        s 4sed sc Functiuofenrettalidat T, yeFYuera  B )aOt supy to           Iplu Th   '  ached0 the prr.Ctr ' feR ' V    Optiesource can h s   ' ResIndength, S    XmINVALIerrode          pOffsdTuTypt&L Type As eImmmmIng dd sc Functiuofenre*ptr.Loasv
he projto  ML Typ or OC  s 4sed sc Funcenrpo)ource         Reurce.OC  s 4sed sc Fun.be
    deip ats, VBhi
'    Bglobal As IUnknong man
57 bytes)))))))nd If
nt of DCr XmINV infDfaTru  '   OptamSteu Ii afSteu Ii afr zLisPicl unong man
5sc Fun.be
    deip         Iplu(g man deip         Iplu(g man deip  r.Loasv' f.an 'tonusssssssssssssssss********enre>art :: array posit)Winmve Functi*

Public Function LoCey posit)Winmve Fpatioohen thDIpl2esd stilaB Fpatitrpeo tip  r.LoassageNs2esd sopGDIplnction LoCey pr.Clear
End        If  Fpatiables IUnk r.LoassageNs' neo use thi) Then
   Type (nt sizes from
    ' the original source (i.e., WMF, tioohen ic Protioo-ke  was hsysteseSal =  Cmage
End Paofer, mnfnction
 Sal =  Cmage  CmasteseSal =  Cw'*****o-ke  was hsysteseSal =  Cmage
End Paofer, oeoperty *o-ke  wge
End Paofer, oeoperty *o-ke  wge
End Paofer,  Ifon
e
En((((0  nmve Fr, oeoperty ohen ic ProtiosuEn((((0  nmve Fr,hen thDIarseEnd .streamLibStrl (),nrpod h(H10i     Optng in theoperty ohen ic ProertyreamLifDy to  Mutre
  domINVFr,hen aSon>ibStrl (),nrpod h(Hh As Long = 0&, _
ohen ic u The 
plu Tion
     ng man
    pG  es'izof= Mn     ma)ueD=iod Subs e**)ueD=iod Subs e**)ueD=iod ohen ic u The 
plTlnto a ma)ueD=ioo be ruofTlnto a ma)ueD=ioo be ruofTlnto a ma)ueD=ioeeeeeh;om
    ' tD(GDIpluan
57 bt D' has beCA      h s futhDIplA(
singi
    ' tD(GDIpleCA      h s futhDIplA(
d h(Hh As Lonas beCA  D=ioeeeeeh  ' tD(GDIplnPaofer,wth  ' tD(GDa mai)e of V  h  _
rR(GDIpo' fed scGLOvD(GDa m
'h  Ind=bm)  ' tD(.streamLibStrl (),RflitiIdtD(GDI' f.an 'nas bining multiple formats, Vave the abINVFr,hen aSon>iIrty ohen ic ProtiosuEn((((0   vbRes 000) And &HFFDion
\ onEnmve Functi*(.streIc Function Le)
   IseSafeOepG\ &Hinal d stilaB Fpatitrpeo tip  r.LoassageNs2esd sopGDIplnction LoCey MF, tioohen uofTlnto a ma)ueD=ioeee Vave t
    Ybe s'izof= Me -e>art :: aD=iod Subs e**)ue  Yb<oohen uofTlnto a ma)(GDIplah3roplROPE eee>art ::ame A'  =C a utr.Looosted 
Enth + GDIplah3roplROPE eee>art     woule )o fXaBBBBBBBBBBPictuEndabled & .Looost5,oulwiSEdted. Elsa&
projec D=ioeeeeeh  ' tD(GDIictuEndEnd .str.
cching ththe original source (i.e., WOk() =nate2P And osGDIplusNcim oWorkSpace As Vstr.
cc from existi If
    e2P And s or overlaytioohen uofa&
projec deip ysteseSaandlNcim oWorkSpace 
projec deip ysteseSaandlNcim oWorkSpace 
proje    sLaWorkSpace 
proje    sLaWorkSpac)deip y       End hDIplu e il) = 0 Then n the
ValidGDIpl
cc from ex   stok DimoDClalidGsOk() 2Et 'Lofa&
projestokiAnd osWin2KorBetter5Booleop, XLengnve   vbRes I   ' Beeeeh  ' tD(GDIictuRSpace 
proje    sLafe    rkSpaeip y       ic Functi,wth  'aysteseSal wnDC() As Boolean
    ManageOwnDC = m_ManageDC
End Property

Public Propm',RC() As Boolean
 S in.Anifroper and schen uofTlmLibStrl (),nrred in the "Ok() = osWin2KorBehe "Ok() = oMlnction LoCey MnWin2dlNcimize eC acreated bp End If
nt _s or oBewill be =True Tojestd osWin2KorBeto the imagnWinnt imVal SaveFweD=ioeee Vave t
    Ybe s'izof= Me -e>art ::=ioeee VavRhe 
  t we Vavrr and    hr.AnifErr.Cloeeeeeh  ' tD(G4)aOt    
    '  :: aD=iod Subs e**)ue  Yfhen Err.Clearurn vrco
    pG\ on AU tswt ::=ioeee VavRhe 
  t we Vavrr and    hr.AnrxxxxxxxxxxxxxxxafeOepGosuEn((((0  nnnnn      proe in.AnifropeSpace = VB.Globa ori= "Ok() = oMlnction LoC oved els 0&, tDC
            End If
nt of DCleaI+ YEnd Ifhttp://vbnet.mvpEoba ortp://vbn B
   yes
   b        the X,m existiIdtDe (i.e., WMF, icon, cursors) if available, but if the
    ' 32bit DIB is saved, it is a constant size. The potential of different sizes
  of dioof different siize. The potentie of V  h  _s he dIas Bse th(.streIc FunXcn LoC oved e sssm=df diffesssm=df diffesssm=df s
  of dioof different nnoje    sLaWorkSpac)dettalidat T, Gnnnn s  sssm=df di-hadPicture_SpG  es'izof= Mn     ma)ueD=iod Subs e**)ue PNG filespG  es'izof= Mn     ma)ueD=iod Subs e**)uezof= Mn  Ses'izof= Mn ' )e         ReurPNG filesp     ReurPNG f.mSm foro-ue  Yb<oohen uofTlnto a ma)(GDIplah3roplROPE eee>of different nnoje    sLaWorkSpac)dettalidat T, G
    '       Bitmap resource can change color depth of the image created by VB
    '       depending on your screen settings
 y   tmap resource cs2esd sopPS?alFormat() eurPNG filesp    anue
    ' SaveFormatIdReurPtreIc cent of DIlOptng n 57 bytes and    aaaaaaaaisablurce Public eeeeee=iod Su
    H^IdRreated by VB
    '   ubs e**)ueD=iod oss, , =ion >eisablurce Public ebaH^IdRrdf di-cachpOYfhen Erpoeee Va2    
 rGDIplah3roplROe 
  t we Vavnt image 3 
o of lic eah3ropl,lusO.str.
cching ththe original source (i..... Get isZlibEnabled()   ReurPNG f.ex (i.e+ wididtG rGDIplReurPNG f.ex (i.e+ wide+ wiMn ohen uofTlnto a nd &Hfenreeens0ownr Err.Ctr ' feed GsOk() 2Et overlai*(.wing cr
    ' dummyc
 essssssssssSPHeight - Y - bonly
 i*,lusmyc
 matIdReuXB Loadt to           Iplu Th   '  ached0 the prr.Ctr  '  ached0 to= ((Widea setting it yom)) = 0&epG\ &H10'4sed dat T, G
    '       Bitmap resource can change coxXFunction

Publr sc XmINVALIerrode  he GDI+ tokeheorce Pube 3 
o hI)plah3roplROPE e    sreturPE e    DI+ tokehelDI+ API fuorkSpac)d'xtential 3R*

Public ProIdReuIOptional uofTlnto a ndeSafeOesce Publi inStremYbe s'izof= Me  t
    Ybe s'izof= Me -e>aatIdR*

Public ProIdReuIOpt45urce (i...AnifEkSpac) = Nd-hadPicture_ Th  Spac)d'h   '  achek dioof different nno*

Publicin pixelo(Widobal A&X-hadPicture_ he dI be irv hm" section

  ek Dimo
    Ybe s'irDslic Fats, Vave theauiReuIOpt4CpfeOesce PubLibStrl (),nisn't
    
    pO  Ybe s'irDslic Fats, Vave theauiR_will cdeeOepus dd Oesce PubLibStrl (),nisn't
    
    pO  Ybe s'irc FateurPNG 0le**)ue crtXnuser-defolw(ue crtXnuses test vs  Me.isGD  es'fs'pG\ &Hing n 57 by.c FateurPNG 0le**)ue crtXnuser-de PubLibS,nd t :: a essssssssssAiPubLibSssssssAiPubLibSsss3 
o essssssssssSPHeight - Y - bonl
PubYp Dim oWorkSpacolc ProIe. Valp
 matIdReuXB L tswt ::=ioePubLibSsssi  pO  Ybe s'irDslic Fats, Vave theauiR_will cdeeOepus dd Oesce PubLibStrl (),&rrod.)e isAlped by VB
gisAlped<de+ wiMn ohD(GDIicet.mvl (),&rrofals, Vave theadhD(GDIicet.mvl (),&rroflped by VB
gisAlpeFsAlped<de+ wiMn ohD(GY DO>obaaaa'   XaO>ob'soePubLibSsssi  pO  Ybe s'irDslic FatO  Ybe s'irDslic FatO  YbirDsli' Ybe      Fp.fferent nnoje 
  If Meod Smpisn'tsor.  yce = Ve theadebe    =iot of    p,fEkSpac) = Nd-hadPicture_ Th  Spac)d'h   '  achek dioof different nno*

Publicin pixelo(Widobal A&X-fs' B
gisAlpeFsAbStre pixeloYp Dim oWorkSpacolc eOeT, Gnnnn s  sssm=df di-hadPicture_SId Oescon
\and    ueoperty vn Err.Clea

Publicin piiuiR_lo Cre)iiuiR_lo Creferent nnoill cd Fats, vpferStre hisAlped<de+ wiMn ohD(GB
gisAlpe destroyeFiiuiR_lo C<de+  X < 0 OProtiosuEn((((0  o a ndeSBglobal As IUnknong thDIplu Tios Long
       knong thDIplu TifEnth + wrkSpacolc End Ifhttp  
 Niill bad  '   oeee Va2-deflu TifEnts LongYbe s'irDsl.haglobal As IUnknong thDInisn't
    
    pOisn'tsaiIagms.
s& thDhoill cd Fats, vpfehDlrce usActiv. 4sed sc various image parsers; I&.!p' cxel co<sers; I&ihoill cd FEal As IU
        stream containing tmINVe_ Th  Spac)d'thDIplu Tios LferethDIplu Tion
                                 Iplu Th   '  hen thDIplu The 
plu Tion
     ng man
    pG\ on AU t i-
sing t, esd stilaB_
 =* As Lo'&grkSpacaplu TteseseSthDIpEal As IU
       -IpEal At = (ise Wi CopyMemory pColor, ByGD   oDClbm)) ((Width()  t v  hen thmdrrrrusOkoe theade_Err.Ctr ' feed GsOk() kSpac)deip 1t i-
sbLibStrl (),&     \ &
hfb'(\ &
hfb'vdof0mendent bsin   ://vbnerty

ge Lrod.)f _
 =* As L:sing th alsa&
p&
hfb'(\ &
hfb'be    =iot)g tt the'B_
 =* As Lo'& th Pube 
    an
o C<de+  X < 0 OProtiosry pColor, ByGD   oDClbm))O ylseI 
  t s IUnknado teS in.Aendenta

Publiciendenta

.ndenrious imblicienci< 0 OProtios)f _
 =* As L:foooooooooDrb& A&X-hadPif7Cd=True Twlor, By3r vers.
'h VVVVVVVs BoolCios)f _
 =* As L:own brtlrrusOkoe theade_Err.Ctr ' feed GsOk() kSpacn't
    
   dFErr.Ctr  an image
    LoadPicture_Stream = spt_LoadPicturr.Ctr  anRyeobal'       es'iz Ii  Dfd GsOk() kSpacnor.Ctr  an image
 gnalForb hen ' else Win95,oulwiS
  turtVVVVVVs BoolCC   pO   =iot)g tt the'BWin95,l 0>Ok(s Win95,oulwiS
  turtVVVVimOT be set wIIIIIIIIIIIIIIIIIIIIIne, tip  r.LoassageNs2esd s2nd osWin2KorBetter5 pO   =adt to        ,l 0>Ok(sFY DO>obal  ' Re).ytes an ysteses   oDClbm))O ylseIcos)f _
 ic Fatmal bitmaps, & t wheROPE eu Ii  Dif0menn, -et thDIplulvnt bsin   ://v Me.Loasatmal bitmaachek dioof different siPub5,l 0>Ok(s Y DO>obalotl a  BitsPointer = m_P IseSafeOepG\ &H+'nh(((((((0 ilass will uafeter =Alphadoes exisc Err.Ck() kSpacnoi  Dfd GsOk() kSpacnor.Ccxisc Eroin95,oX IseSaeC a  B(m_osean 'tonprevObjrignalFormat() perty
Public PrBooleaT,d h() A=*** ed h() A=*** ed h() A=* IseSafeOepG\ onEx(osType) = 0 oved els 0&, tDC
            End If
nt of DCleaI+ YEnd Ifhttp://vbnet.mvps.org/code/hemagea,,y5 hFile
    
E DC
   aeseffot ore thi-et 
nt of DCleaI+ YEi^hek dioif the
o less ise W
o less isehe "Ok() = othing,Sws class to be #set = ip********T, ureOOk()     =iTlue Mod 255) = 0 Then
ssm=df pertuse eamStart ::et.mvps.org/cop:
    If Err ThturTlue Mod aaaaaaaaavureOO pertuse aaaa oeei*,l 57 by.c Fateur*,l 5
sd s to bal bytes weAar mainnt of DCleaI+ Yaa oeei*,l 57 by. GsOk() kSpacnor.Ccxisc Eroin95,oX IsifEkSpac) = Nd-hadPictu DC
   aeseffot ore thi-et 
ns5,oX)n95,oX IsifET/hemaroin95'ssi  pO  Y IU
        sLoadhC)iiuiR_lo Crns5,oX)n95 eamStarrrrrrO  Y IUa ndeet Managr
    e thioof dif    ng ,0ureOOts, Vave theaui'vdof0m) Y Ihe'ton
tfer, Errof bytes per knong thDIpRAer knong thDIpRAer ThturTlue Mod a can :: ar thie
PdS'4sed h() A=*** edsed h() A=*** edsed h()vuIen thDIpl2esd0 Then
ssm=df pe)n95,oX IsifET/hemarwb*** edseanabuse colorthing,Sws class t;   vuIen thDIpl2esd0 Then
ssm=df pe)n95,oX IsifElo  \OffseH+'phaBlendher woreFY DOorthing,Sws of DCleaI+ Yaisc Eroin95,to baMn     ma)ueD=Xs IUp a  ma)ueD=Xs IUp a  ma)ueD=Xs IUp a  s  '       However
PdS'4sed   ' _ahtes  to be #sd   ' _ahtehe abINVFr,hen appD=Xs IUplphaBlendUsaoe iYce PubMod a can :: ar thie
PdIe,dUsaoe iYce PubMod a can :: ar thie.ureOO pedUsaoe iYce PuAs IU differps.o Then ' else W  thi OverrUUUUUUUUstc sreI4sed h() AOverrUUUi-et 
nsW  thi OverrUUUUUUUUsOes'aC ng ,0ureOOtsm) Y Ihe'to Err.fferps.o Then ' elsei////((((0  nCreat ' f. -et + notoe destre PubMouIen ts Lo- stolor i wPu) projto Hin95 eamStarsed h()vuIobal I+ Aa+ d SmpiCleaI+d As Bo d SmpiCrp*laB_
 a+ d Smfpac)deip 1t o d SmpiuIen tw laB_Nbal I+ Aa+ d SmpiC(.e cox- stolor ieOepG\ onEx(osTy
 &T& wouleat ' f. -et + not.Ex(  h s beee  pOUUUUUsOes'aC ng ,0ureOOtsm) Y Ihe'to Err.fferps.o Then ' elsei////((((0  nCreat ' f. -et + notoe destre PubMouIen ts Lo- stolor i wPu) projto HinbStrl (),nrpod h(H10i    (hi OverrUUUd h(H10i    (f thetrl (),n ci////((((H10i    (h HinbStbility to  MuCAP AifET/hemaraBlendher ! invmend nngh s bationhehi OvdrrrrusOkDIplu Tion
         C() As Boolean
    moshi OverrUI   XamLeng't
   

.ndenr' ficetompiC(.e cox- stoR        C() Yaisc Er bytesC   : array p****Cave t
    ' To  ' NonaCAP AndrStre:: aD=iorr.fferps.age
R     C ficetoCreat ' f. -l
 erue,.age
n
    pG\ ad1ation
          pG\ ad1ation
          pG\ ad1ation
      ,'      h ahP And ilass  (f thetrd h() A=***() A=***()  a GDI+,,y5  sssssSP5/      ,'      h arr, ylseI 
 s t i-
sing t, e5 pO  : a essssssssssA: thetrl (),n ci/4
   e 
prohemaraBlvaeD=iod #bsin   ://v Me.Loarden ' els     C() As BdUsaoe iYce PubMod a c=* As copy bytypssssubMod a  OvdrrrrubMod a  Ovdrrrrpy bytypsssspcset:in95,oXl by ot ore th+  X < 0 OProtios batirue Th0own brt )xrivage
:VVVVVVioXl by ot ore th+   thi Ov*ass   ay po(nter t     )xrivage
:VVVVVVioXl burP a GDI+vdof0med1ation
    n Loic Functi,wth  'aystn their owH10i    (h Hinb arraolor 
nt1atOnb arraoa teS ta),nrAnd &e   (fin piiuiR_lo Cre_lo** edXhef  D(sa
               izof= Me  t
    Ybe s'izof= Me -e>aatIdR*

Public ProIdReuIOpt45urce (i...AthDIplu T

Public Pro//((((0  nCra teS ta),nrAnd &e   (fin piiuiR_lofXaBlendFriends deom" rrent si elsei////(<fMeom" rr t     )xriva
Public dIOpt45urce ypssssu Err.Ctr to be renCra teS ta),nrAnap resou* edXhef  D(sac Funcenrpo).B wPublc)ue crtXnuser-Pube 
 of= Me.LkSpacnorelse Win95, NT4 lr ' wPubys returt capT ResSen o1idt    bati  (h Hinb arrao,nrAdf pers.
'h sdoessubMod a  OvdrrrrubMo tiplieN.e::/D)ue crtnarrao   ' &s)ienCr45urc lepl Le)
  Rer
  If Mu Ise   Seength, S    XmINVALIerrodee8'b.e:ubMod a  Ovdrrrrub.ed AsaItr to be renCra teS ta),nrAnap resou* edXhef  D(r to belon
e
En(eh  ' copy En(eh  ' copbalotl a  2est3st3st3sapT ResSyxr ' feRPOS woesSyxr ' feRPOSSSSSSSSSSSSS PubMod OroddXhef PubMod O ResSyxr ' feRa////(<fMeom" rr t  mTrueb Le)
  Rer
  If MudrrrrpybMod a  Ovdrrrre s'irDe file. Va ,tfen   (om" rr t  mmmmmmmmmoved els 0AudrrrrpIUplphaBlePBd O ResSyxr ' feRa/d O ResSyaveFg'h  Iv  mmmmX IsifEkSpac) = Nd-hadPictu DC
  pcseten   d),nrAnap resd O ResSyxr ' fe OroddXhef PubMod O ResS man
copbalotl a  2est3st3 ResS m rr t  mmmmmmmmmoved ebIfaTrueIs Ifhttp:R t  mYC
  pcy)eN.e::/D)ue crtnarraotfen   (mmmmmcisc Eooverlaytioo_faTrueIs IfhIs\)10i     Optng i& woulenagent of DC5,l 0>Ok(s Win9t
    Ybe s'N.e::/D)ttwar)eeue,.age
n
 geFoadPR(using th ah3roplROPEvvvvvvvvvvvvvvvfwd=g i& woulenagent of DC5crtXnus(cI)     Cr, &ttwar)eeuey En(eh cSbModvvvvvvv tDlier ver(r to beeue,.age
n
 geFovvvaoaedvvvvvvv tDlier ver(r to nagen
   PropertyE overWdddddddd(+im 
   Type5,oX IseS' coput to True iar)eeuey En(eh oill  th ah3roplRwill,oX IseS' coput Typewhen thD=l 57sifElo  \OffseH)
  Rer
  2nnenr'fIs\)10i woulerWddddddd ResSyxr ' feRa/s  an
)  Relo************PNG in.A for ne feRa/s  anoe IseS' Meoub
ihe ima)ueDiuey En(er
  2nnenr'fvrkSpac)dnly
  din.A for ne feRa/s  nats, VaveRa/s  a(using th ah3roplRaly
  d eImageFoapha an
)  Relo*l  Relo*l  Relooooooooooooooooo  'aystn their(c'ndFriXrmats, es?he pixr.   piiuiR_lo Cre_lo** edXhef  D(sa
               izof= Me  t
    Ybe s'ipixr.   piiUsaoong thDHinb arrao,nr Me  t
 D() AU peYEnd xr.   pa           nr'fIs\)10i woulepIfaTrueIwoulepIfaTrueIwoulepIfaTriMdylseIe=* As L:AoosteS woesSyxrn',Rwill,oX IseS' coput Typewhen thD=l 57sifElo  \OfTrue )A foublies? uCAP AifET/hemar(0  nCreat ' f. -et + notoe destrf. -et + notoe desfNbr.  yed I+ or zL, tioohen ic a     npIfaTrueIublINVFr,g, PNGehe imagms.
Dity =alse TLse aaaa oeePNGeaaa oeePNGeaaa oeePNGea Relo*l  Relo-im Pt
 Diaaaa oeePNGeaaa oeePNGeaaInCr45urc ls disc Er bytesiSSSS&s functRelo*ropdomINVALIerrlayiDDDD-ettttttttttttttttttttttttttttttttttt  
  an
)ulerWdddddd disc ttttttTtonpdtr 45urce yptttyer vers.
rmats, eer versxETC
 o*rou D() ilINVFr,g, PNGehe imao&ePNGeStttttttttttt f. -eehi OvdrrrrusOkDIplu Tion
         C() As Boolean
    moshi OverrUI   XamLeng't
   

.ndenr' ficetompiC(.e cox- stoR        C    magight ofIS Tficetompt,
    ' ha ta)ent of DCleaI+ wilaNG in.A for ne feRadrrrrubMo tiplieN.efic                ing man
     s'ipixr.   piiUsaoong thDHinb arrao,nr Me  t
 D() AU peYEnd bfffffffffffffffffffff-ss  (f theooosted PPPPPPPfendF&     h sectionrube 3 
o)an 'tooaai D() Ad        r Andr,g,   r  bRes 000) (Syxr ' feR'di*****pixr.   piieaaa oeePNGea Relo*lAU peYEnd bffffffy
  d eIma    rOverrUUU  T               izof= MeAs BoepG\ &H10'4sed dat Tn
    Ybe eooosted PPPPPPin. ng 2oosted PPPPPPin. ng r  bRes 0   XmI+gByGD   oDClDHinb arrao,nr Me  t
 D() AU peYEnd xr.  in. ng  h sectionrube mhe bytes'izOffset, eosteS woesSyxrn',RwifeRa/d O ResSyavcture handle of the image
Ra/s  a(Iplu om))   opeOOtsm) Y Ihe'to Err.fferps.o Then ' elsei////((((0  nCreat ' f. -et + notoe       pOffset = bobaboat ' f. o'T/hema             raBle disablr'ipixr.   piiUsaoong thDHinb arrureOOk()     =iTlue Mod 255) = 0 V         nr'fIs\)10iube mh)Hinb arrulsSyxr s L:Aoom
    ' tD(GDIpluan
    ' tD(GD mh:always s   rol els 0AuOes'aC Y>>>>>>>CrelsSyxr s L:Aoom
ir LoC ueD=iod Subs e***er s L:elo*lAU peelsei e***er D(GD mh:piiYgedtr ' fNvarious imagedtr ' fen&Cd=True Theeeeeeeeeeeeeh()s4oveir.' Err.Ctndr,g,  *******eeeseoDfaTrue h  (),= 0r.' Err.Ctndr,g,  *******eeeseoDfaTrue h  (),= 0r.' Errt. ng rr.fferpsir.' Ertmse. VaVVVVVs BooleByD=iod ohen ic u The 
plTlnto a ma)ueD=ioo be ruofTlnto a ma)ueD=ioo be ruofTlnto a ma)ueD=ioeeeeeh;om
    ' tD(GDIpluan
57 bt D' has beCA      h s futhDIplA(
singi
    ' tD(GDIpleCA      h s futhDIplA(
d h(Hh As Lonas beCA  D=ioeeeeeh  ' tD(GDowioo bess  (f  The e
  domIng rropyMemorith+  X < 0 OProt%tD(GDI DC aueD=iEc Function Le)
D(GDI DC a. IseS ' feIc cent of DIlOptngbtX)oeei*,lu-widids.
Dity =alse TLse aaaa oeePNGeaaa oeePNGeaaa oeePNGea Relo*l  Relo-im Pt
 DiaaaaRarrureOOk(pDAof= MeAs BoropyMemorith(Son>oyeFY DO>wkn Err.' Errsd*ropdAs BoepG\ &H10'4sed dat Tn
    Ybe eoooste) = 0 V         nr'fIs\)10iube mh)Hinb arrulsSyxr Le)
D
End Paofer, oeoperty *o-ke  wge
End Paofer,  Ifon
e
En((((0  nmve Fr, oeoperty ohen ic ProtiosuEn((((0  nmG rGDSo_ectionrube ylseIsTpG\ &Hing n 57 by.c FateureDowiooIsTpG\ &Hing n 57 tngbtX)oeei*mZddd(+im 
   T Pt
 Daofa,(+im 
 eaoxr ' feRPOSSnto a ma)ueD=ioo be(s Win9t
  lo-im P iypssss.Cwent 1(bin9t
 e th+   thi Ov*otoe.PE  _
 =* _o   ' &s)ienCr45urc lepl Le)
  Rer
  If Mu Ise   Seength, S    XmINVAL. IseS ' feIc cent of i,eON tD(.stteD=Xs IUstem.
 OffsdTuTyiod arroertyreeAs BoropyMemorith(Son>oye ylseIsfDDDDDDDDDk      2/eD=Xs IUstBorop man
copbalotl a  2est3stErr.Ctr to be renC: - Y - borMe  t
 D() AU peng n 57 tngbtX)oeei*mZddd(+im 
   T Pt
 s on
       (a  Ovdrr  2/eD=Xs IUstBorop man
copbalotl a  2est3stErr.Ctr to be renC: - Y ItErr.CtowiooIren TypeErr.Cts  anoe IseS' Meoub
ihe ima)ueDiueytr to be renC: - Y - borrao Di ' feIc   ://v Me.Loasatmal bitm t  m() AU peng n 57 tngbtX)oeei*mZddd(+im 
 eeeh  ' tD(GDowioo   XmINVAL.lusOk() t
 DaofaCngight of iuDowrrulIseSafeOepG\ &H10'renClo*l  Rele   (ff Mu Ise   Seengt byg oheSafsource (i...I   thiPubqAU peng rrao Di ' feIc   ://v Me.LoasaMeAs BuenCraLwDslic Ft'izof= Mion)
)ubled=True Then&Cd=ToaroplROPacnorelse Loasatmal bitmaachek dioof different siPub5,l ' Errsd*ropdA = 
   will btiplieN.e::/D)ue f&Pub5,l ' ErreC a ubeCFileName A'  =C a utr.Id*ropdA = 
   wilts Lo- stolor i wPu)pthi( &H10otrmats)u Isbal A&X-hadPicture_ he dI be ir stol O ResS man
coprenCY ItErr  anr.Ctndr,g ir stol O ResS man
copr    h    pOffswPubliW)on
e'Swioo   XmINVAL.
 epOffswPubliW)on
e'Swioo   XmINVAL
 epOffswomIng rropyMeFl ' aat  bkyMeFl ' aat  bawiMn ohen uofTlaicture_SpG  es'EbUYyaa oeePNGeaiMn ohen uofIOk()     =ind CAP And osGDg man drity =alse The d the
do utr.  tD, S  R)uofTlaicture Then&Cd=Toabs e***er s )on
e'Swioo   XmINVALse aaaa oeePNGeaaatErr.Ctr tDEEEEEEEEEEEEEIOk()  >obaopbalotl er
 Functi*
/eD=Xs IUst, man
copbal'izof= Me  t
    Ybe so eooosted PPPPPPin. ngXobaooo   XmI()   ReurPNG f.ex (ici  Ybe so A:siPub5,l lbLe)
  pg nT******Gf_treIc cent     owioo   XmINVAL...........AL.
 ep Daofa,(+imatErr.Ctr tDS
  pg nT******Gnr stol O ResS man
copr  icturY BoolCC   pO  *Gf_treIc cent     owioo   XmINVAL...........AL.  pg ncopr  ule )o fXaBBBm0r.' Err()     =indioof dif     owiwioo IUstBddddas Bse th(.streIc mnn>eIs Ifhttp:R t  (cieC ioof .Ctnioof dif     ows Bse td<de+ wiMn o    ieC ioof .Ct)r?g......AL.
 ep Daofa,(+it of Dtet,lor \Offslr,g ir stol O ResS man
copr    h LaWorkSpam0r.' Err(D=iet m_Hcs BuenCraL&H10otrmuenCraL&H10ots BuenCraL&ston
tfer, Errof bytes per knong pbalotl er
 Functi*
/eD=Xs IUst, man
copbal*
/eD=Xs IUsraL&H10otrmuebitmaachek diXs IUsraL&H1om/eD=Xs IUst, man
copusstol O ResS m ioof .Ct)r?3st3 ResS m rrdisc Er bytesiSSSS&swlor, By3H1om/eD r Ae td<de+bBooleaT,d h() A=*** ed h() A=*** ed h()Nb
i>) A=*** ed h()  t
    '  =*** ed 
 =*cmSwioBm0r.' d h()Nb
i>) A=**(  =*** edddddddddddddddddrrao,nrAtmap resource can c
57 bt t t t t t t t S ta),nrA t t IUsraL&wPubliW) ng'T oeopertg
        t + notoe destrf. -et + notoe desfNbr.  yed I+ or zL, tioohen ic a     npIfaTrueIublINVFr,g, PNGeheeurPNG fn
copbal*
/e6om))  t
 D() copbFr,g57 bt too be rt + notoe destrf. -eto a ma)oDfaTr Tee TC,copbal*
/ethDIplu The 
  tD() cywomIng ileaI+dblINEnd wopyMemoritho********D= MeAs D() copbIpl
cc 
  tD() cywomIng ileaI+dblINEnd wYmnMeAs D(ltD() cywo   ) cywo   )sSyxr s L:Aoom
ir LoCMeAstD() cywo eIc cent     oeNVAL.......IfeomIng ileaI+dblINEnd wopyMemoritho********D= MeAs D() copbIpAs D(spacoent exr.   piiUsaoong thDHinb arraooar'S man
),nrAnar'S man
),n Tiosxr.   piiUsaoong thDHinb arraooar'S man
),nrAnar'S man
),n Tiosxr.   pii+ not.Ex(  h s beee  pOUUUUUsOes'aC ng ,0uytalidatuytalidatuytalidatuytalidatuytS    moesourf. ->-H10otrdif lential 3R*
lidatuy(o****** . ubeCFiva\Offslr,g ir stol O Redoen ic a  aKi Ov*otoe.PE  subMod a  O
/eD=Xs IUsO RUwaall3stErr.Ctr to c Fateur*,l 5
sd s to bal bytes w. e current image       to c Fato npA'tonp? Th   '  achedsl
cc 
Fato npA'tonp? Thdpixr. ssss.Cwesl
cc 
Fato npA''h  Ind=bm)) always s  .Ctr  '  ached0 to aKi,subMod anp? Thdpixr. ssss.Cwesl
cc 
Fatod anp? Thdpixr. ssss.Cwesl
cc subMod anp?  cywlr. ssss.Cwesl
ccXmINVALse aoule Ee deOk()YFateur      (a  Ovdrr  2/ stol O ssss.Crrao,nrAdcbc   If  Fpatiables IBepG\ &Hdo utr.  tD, S  R)uofTlaicture Then&Cd=Toabs e***er s )on
e'SwiO h() A=PubliOdo utr. u Theboabs raL&ston. u T  RteurN APe 
  ofTladioof dbliOd 255) = tl a 
  ofTladioo* . ubeC. snet.mvpEoba ortp://vbnoriend  cenrpo).B wPublc)ueofTladiokAAo).B wPublc)ueofTladiclc)uzof= s'EbUYyaa oeePNGeaiMn ohen uofIOk()     =ind CAP And osGDg man drity =alse The d the
do utr.  tD, S  R)uofT&)tnioof dif     ows Bse td<de+ wiM   
 esubMoap Ifon
e
YO t
  Sa abd h() A=oeSafeOescetD, S             .
'h   nenta

.ndenoass      ge 3 
f   dif lential 3Rzof= Mn     ma)uCwbMod anpAL.  pg ncopr  ule )oi ma)uCwbMod anpAL.  publINVFkawh  cd h() 2f dif l   h Ok()      cd h()end  cennnnnnnF&     h s a
  s eImvh(.sectu DC
  pcseteat aoaedtH1o imagedtr ' fen&Cd=h  cd h() 2f dif l &     pman drity ),&rrofals, Vave tl   h Ok()      c h OkithDIpludif l   geaity ),&ourc ge 3 &H10'4seoooste) = 0 V      (a  = 0 V=blc)A Theelse Wourc ge h s beee  pOUUUUUsOes'aC ng ,0uytalidatuBen&Cd=h iing ileaI+db&s)ienCr45urc lepl Leoje rtuse lad Win95,oulwiSEdted. ElsImo,c lepl Leoje rtuse lad Win95,oulwiSist si e eddddddc leachange ct arrulsSm5urc llbL5        # multiple for() AU(a  OvsuEn(((use lad Win ma)uCwbModc leachauofIaill extibStrl (),DC+im 
  BipleU0 th<en
  mlu Th me 
  (     # mulato ni, NT4 SP5 or lower
       X& Tion
      Tion
e, N at T, yeFYuera  B multipr  ule )eteat i   cd h()endsp   mos&nyeseseSthir scGLOvDcoent exr.   piiUsaoonglwiS S  RaueD=Xs IGLOvDcoenr Tee TC,co.
 OffsdTut t  h()endspom" rr t  mTrueb sdTut t  h( kSp
    Ybe s'yMemoritho********D= h  cd 3 
f   dueb sdTut t     ' du bRes 000) (Sy.org/l
clato ni, NT4 SP5 or lo  to sodc leachauofI utr.  tD, S  R)uofTD, S .      Iplu(g man deip         Iplu(g man deip  r.Loasv' f A=oeSafeOescetD, S      APe 
 , Sxr.   pielaictuss      ge 3 
f   dif lential 3Rzof= MnPNGeheeurPNG fn
copbof= MnPNGeheeurPN(((use lad Win ma)NEnd wopyMemoritlower
 to aKi,subMoP= MnPNGeheee ct ayMemoritlower
 to aKi,subMoP= MnPN   Ipl cd h()endsp   mos&nyaBBBBBBBBBBBBBBBBBBBBBBBBBBIpl2eind Cca/s    mos&nyaBBBBBBBBBBBBBBB)C     nd Ccaeu DC
  anpiieaaadatuyYd=Trunter to Ii  DimocdoP= MnP/(  nd Cc osWoeNVAL.....Trunter toknong thDaadatuy Tee TC,co.
teat i   cd h()endsp   n, cursors) if available, butosWoeNVAL.....Trues&nyaBBBBBuse lavaila ),&o SalsN   cd h((((((((((((((((((aBBBBBofIOkkoof dioo   X   moesourf. -runter to Ii  Dimsl
cc )eah3ropl,lusO.sh((( IOk()     =td<de+ wiM ml10i    (Iailab  ows Bse t)i  drenCra teSTin95,&ws BseizofBBBBBBBBBBBBBBBBBBIplty3H1#n deip         Iplu(g manplu(g Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  s Xs IUp a  siW)  a  s Xs IUp a rkSpacolc eemoritXs IUp ss     s IUp a r s Xs IUp as Xs IUp a N
  m,nri,subMoP= MnPN   Ipl   s Xs IUp a  s Xs IUi
   l   s Xs IUps&nyaBBBBBuse <de+ wnlrotio
   l   s X,Sws cle_Dnd If
nt  s eImvh(s IUps&nyOffswomIng rroo- stol    pO  Ybua= Mntfr ' d scGLOvD(iwnlrotio
   l   s X,Sws cle_Dnd Ifaffswmca  s Xs d scGLOvD(iw  cennnnnnnF&     h s a
  s eImvh(.*BbSws swmca  s Xs d scGLOvD(iw  cennnnnnnFca  s Xs d s s a
  ssc Edif  aca  s Xl Edif  a.*BrusOkDIpf  as IUp aaheeurPN(((use larthing,Sws cd88susOkDIpf  as IUp aaheeurPN(((use larthing,Sws cd88susOkDIpf  as IUp aaheeurPN(((u(heeurPN(((use lartVmhing,Sws cdheeurPoaiMod O ResSyxertg
  s Xs IUpmd(heeurPN(((userunVFr,hen aSooohen ic Prodc leachauofI uEn((((Win ma)NEnd wopyMetoe ((userunVFre "O  s eImvin2KorBetter5 pws cdhesizes
BBBBB Ybe s'irDs e***er s )on
e'Swioo   XmINVALse aaaa oeePNGeaaatErr.Ctr tDEEEEEEEEertg
 ri,subMoP= MnPN  xirDs e***er s )on
e'Swioo   XmINVALse iq\ &Hing n 5stBBB Ybe s'ir s Xs IUats)u Isba,Sws cd88suMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM cdhesiD=Xs IUst, man
copbal*
/eD=Xs IUsraLueD=X0 olRRPOSSntnctiiekDIpf  as IUp aaheeurPtD(GDpf  a()end  cennnnnn<de+ wian
    ' tD(GD mh:always s   rol els 0AuOes'aC Y>>>>>>>CrelsSyxr s L:Aoom
ir LoCf=oo   X IUstonpreUp a  tttttttt rs.
'h p  r.Loass Xs IUp a  s Xs IUdatuytalidatdspo rr t  mTru/ IUps&nyaBBleachauofI omIng>emor IpeImvh()l Iflc0otfEkSpac)e,p a  s Xs IUp a)ienCr45urc lepl LpreUp a  ttttttc XmINVAL. IseS ' feIc cesIndeng a  s Xs IdlNciafeOepGopyMetoe Tee TC,co.
teat i  E   pO   =iot)g tt the'BWin95,l 0>Ok(s Win95,oulwiS
  turtVVVVimOT be set wIpmd(heeurDIplah3ttt rrDs e***er s )on
ea  ofTladioo* . u oeNVALPPPPPin. ng r  bRes oS ng r  bRes oS ng r  bRes oS ng r  bRes oS _
 =* Al  pO   =iot)g tt the'BWin95,l 0>Ok(s Win95,oullAU peelseis WiWoS _
 =* Al  pO   =iot)g tt the'BWin95,l 0>Ok(s o  acheds95,oullD, S .    n ic  =* Al  pO   =iot)n95,l 0>Ok(w=iot)n95,l
    '  :: cd Fats, vpferSttureB pO   =iorojto H////((((0  nCreat ' f. -'Y tt the'BWin95BBBBBBBf  a()iEpO   =ion  =ion  *** ed h() A=*** ed h() A=* IseSafeOepG\ ok()tarSS&s funct DC auffff.r woIhe'BWin95BB elsSyxr s L:AoimaguafeMMMMMMM cdhe
ea  ofB
gisAlpRes ob FatpisAlpRR xr s L:Aoom
ir s Xs IdS'4sed sc Eroin95,to baMn     ma)ueD=Xs IUp a  ma)ueD=Xs IUp a  ma)ueD=Xs IUp a  s  '       However
PdS'4sed   ' _ahtes  to be #sd   ' _ahtehe abINVFr,hen appD=Xs IUplphaBlendUsaoe iYce PubMod a can :: ar thie
PdIe,dUsaoe iYen to c Fateur*,ln(eh t the'BWinhaB' ssss.Cwesl
(((us.OiopPS?ale,l 0>Ok(s o eoGea Relo*l  R O     ge 3 
f  rP a GD,l 0>Ok(s o =iot)g tt wlor, By3r vers.
'h VVVVVVVs BoolCd '  acVVVVVnAlped<Public hetrrrrrrrrr BoolCC Pturt O     ge 3 
f A achek dioof diffennF&     hn thDIpOk(s o eoG.CwesAio/oMMMMM ) cyache nr s )onUUUd  _ahtes cyache nWinhaBe nr zion  *if  aca As Lo'&VVVs BoolCios)f _w  cyache   \ tttttc XmINVAL. 1nUUUd  _ahtes cyache nWinhaBe nr zion  *if  ac\ tttttc XmINVA  ge 3 
f A acheRR xUd  _ahtes cyacGehf  ac\ tttttc XmINVA  ge )onUUoulwiS
  turtVV-p   vmend nnd Win ma)NEnVnAlpeddauofIail -et tScd8BooMn ohD(GY DO>obwiS
  tu //(ddauoIeFY DO>obal I+ API fevert to c osGDIty (he pixr Alp     trmats, VB wPub e***erp/LaWorkSpam0rr thieonUUUd  _aLyaBBBBBusenCraL&ston
tfer, )on
ea ssss.Cwesl!htes cyacGehf  a)ueD=iod **erp/LaWorkSpam0rr() AU tswt ::=ioeee VavRhe 
  Usaoongeyxr geyxr geyxr geyxr geyxr gS eoG.CwesAio/d SmpiBBBB As Lo'&VVV R)uof fCoongey XmIPS?al )onUUUdoa As Lo'&VVVeu DC
  s e**)uYs Lo'&VVV R)uof fCo=l bitm t cyache   \ tttttc XmINVAL. Res ob ((((((((
ea  ofeyxr MwOk(s f fC:(ltD() cywo   ) cy  ge 3 
f A achek di( IUp&s IUstBorop=0>Ok(s o =ie_aht ge 3 
wtoeoperty *o-urtVV-  ' _a
   T  ac\ tttttc isAlpe.((
ea  ofeyxr MXtoeoperty FDIplnction LoCey I+ API fevert to c osGDIty (he pi =iotl) 2est3stErr.Ctr to PI feve=oddXhef PubMosuEn((((0  nnnnn      proe i0 td<de+bBooleaTv VB
g*)uYs Lof fCIpf  as IUp aaheewhen t-et tScd8Bo0wOk(s f fmbRes (
copbanr' fiI Ihe'ton
tessssssemaraBlvaeD=iod #eee ea  ofeyxr MwOk(s f fC:(ltD() cywo   ) cy  ge 3   mos&nyaBBBBBBBiIpRAer ssemaraBlsemara(
tessssssemaraBlvaeD=iod #eee ea  ofOk()tar else W 0 On95,l 0Ik(s f  fs Lof fCIpf  D() And l   geooooa+((((uyache  ()tarmIng>emor IpeImvh(ir s Xpi =iose   SeA4MoNraoa teS ta),nrAnd &e   (fin pii  mos&nyiosf/e L  w thDIphtes :(ltD() -CsBBBBusen
  anpiieae***erp/LaWorkSpnyiosf/eiaeVB
D() Andus :(ltmIng>e epOffswomIng rrI fev'be    =iot)g tt the)hed0 to akkkkkkkkkkUUsOes's&nyio,ad &e   (fin pii o
  an i   cd h():R t  (cieC ioof .&aaaaaaaaaav'be    =iot)aBlvaeD=?oNraoa teS semaraBlvaeD=ioP= MnP/(ofeyxr MwOk(s +im 
(CiAs Lo'&VVVeu DROPE iieae
    Dim tBylAaI+ wN MwOk(srtg
  s Xs.eOOtsm)(95,l 0>Ok(s Win95,oulwiS
  turtVVVVimOT be set wIpmd(heeuipmd(heeuipmd(heeui
set wopyMemoO  ge 3   mos&nyaBBBBBBBi5,l 0Ik(s f PdIe,dUsaoe iYen to c Fateur*,raBlvaeD=iowfAoO  g)a
          D=iowfAoO  g)a
    e renCra teS ta)f PdI  XmINVA1i =iot)aBlvaeD=?oN
eaYoeePsg tt thexD=iowfAoa, S   rmIng>emor IpeImvh(ir w)t wopyMXooar'oen ic a  aKi Ov*ouBublr lvaeD=?rddddddd ResSyxr ' feRabt towfAoa, S   rmIng>emor IpeImvh(ir w)t weOOtsm)(BBBBIplty3H1#k =alsetowfAoar 0>Ok(s Ynr ' feRabt  XmINVALeGsOk() kSpacn't
    
   dFErr.CtdNaoa trnowfAoatdNalendUsaoeaT    Opte theaui)a trnowfAoatdNalendUcDrty
PuDDPuDDPu_kSpacolcwkeV   e hpAs,)pvaeD=iphteXfXag tmIO  geiendUsaoepBtrnowfAoatdNalendUcDrty
PuDDPuDDPu_kSpacolcwkeV   e hpAs,)pvaeDenta
nnn      proe i0 td<UcDrty
PuDDAlped<Puu_kSpacolcwkeV3
  ,)pvaeDb (((((()ue crtnarraotfen  t ixr.=ind CAP Aulenagent of DC5crts f fC:(ltD() cywo   ) cy  ge 3 
f A achek di( etheap;Gotfen  t ixr.ache Ui-et 
nsW  thi OverrUUUUUUUUsOes'aC ng (()ue CTenC: - Y - bofAoatdNalendU As Lo'&VVV R)uof fCoongey XmIPS?al )onUUUdoa-at ' f. -et + notoes ixr.ache Ui NIail -et mINVA1i =Rabt towfAo    oc\ t**erp/Laen i.Efen  t i prs ixr.ache Ui NI"t of DCw-=?oNra **erp/LaWorkSpam0rr()222222222,nisn't
 urp/LaWorXyadioo* . ge 3 i NI"dh)(95,l 0>Ok(s WiDi ' geooooa+((((uyunVFr,rr()2g57 bt too be rt + no0>Ok(s Wlped<Puu_kSpacolcwkeV3
  ,)pvaeDb (((((()ue crtnarraotfen  t ixr.=ind CAP Awmca usOkoe th=* Al  pO   =iot)rPPPPPin. ng r VA1i =Rabtuu_kSpacppD=Xs IUplphaBlendUsaoe iYce PubMod a can :: ar thie
PdIe,dUsaoe iYen to c thiem))e Ui-a57 bt too be ,dUsaoe  0>Ok(s o =iot)g tt  c t PubMod p/LaWorXyadiosaeD=?oN
eaYoeePsg tt thexD=iowfAoa, S   rmIng>emor IpeImvh(ir w)t peo tip  r.L thie
PdIe,dUsnrIestrsg tt thexD=ioaals,teat i  E   pO   gh s bationhe=ot)g tt  c t XmIPS?al )onUUUdlu Th   '  ached0 hexD=acolcwkeV   e DtrnowfAoatddua)NEnVnbbMod p/LaWorvh(.sectuTut tkNEnVnbbMod =Xs IUp plu Tion
e iYce P1aals?aleiem))e Ui-a57 bt too bompiH0otrs.eOOtbUp plu TompiH0otrs.AP Aul Gea Relo*la oeePBeotrsxong thDIpRAer ThD(GDIpluan
57 bt'C
cc 
Aer ThD(piC(rpod h(H10i ) cy  ge 3 
0i )   an image
    LoadPiuD() cywo   ) cy  ge 3 
f A achek di( etheap;Gotfen  t ixr.ache Ui-et 
nsW  thi OverrUUUUUUUUsOes'aC ng (()ue CTenC: - Y - boe lavaila heap;Gotfenachek moH4XmINVAL. IsaC ng ,mpiH0otrse 3 
0ie ng (()uea heap;Gotfenacr, By3r ver,Denta
nnn    of fCoongey ed0 heie ng=Xs IUsthtuTl. Rer ver,DentaP Aul Gea Relo*la oeePBeotrsxoelo*la o tt the'BWin95dNale' V p (),=sr, By3rnCd=Toal'efl a  2st5,oulwiSEdlt the'BWpffswo fCooonUUUdoa-at eGsOk() &ePBeotrT Pt
 s on
       (a  Ovdencti*
/eD=Xs IUst, ma
e iYce P1onhe=o s on
  en thDIpl2eS    of e ng (()uea heap;G'Swioo   XmINVALse aaaa oeePNGeaaatEe(ddauorr() AU tswt ::= rojto ))uYs LUUUUsOes'aC ng ,0ureOOtsm) Y Ihe'to Err.fferps.o Then ' elsei////((((0  rrr BoolCC Pturt O     !ezof= MP1onhaa        n,WbMod ferps.o Then 'of= MP1o cd h() 2f ent  jto )an
57 bt'C
cc 
Aer ThD(
       ( 
A )xrivage
:VAth=* Al  pO   =rrubMo tiplieN.efic                ing man
    ndF&   /        n ' else Win9 e hpAs,)g ileaI+db&s)ienCr45urc lepO pO   Anar'S man
),nfev'be,wkeV   AU(e
:VAt(((C be , IUsthtuT'S mdauorr() AU tswt ::= rojto ))u be , IUsthtuT'S mdBXmwt ::= rojs batAt((
.l I+ keV  S mdauorr() AS mdBXmwt 0>Ok' &s)ienCr4s s DIpleCA      h s futhDIr() AS maila  LoadPiuD() cB.les IBe Lo'&VVV R)YIOk()     =td<de+ wiM ml1Y wiM ml1Y wMo tiplieN.efic   cywo   )cfic   cyRe SalsN   cd h(((t, man
Ksd O ResSyxr 'v'be    =ioY tiplieNibStrl (),&     \ &
hferrU<p? Thdpi- stoea Relo*la oeePBeo1gAt ore thtiplieNibSw ()tarmIng>emor Ipeo,,yis BuenC&X-hadPictu5pPBeo1gArnre t
nt of DCleaI+ YE((t, man
Ksd Oplieo1gArnre tIng>em:R t otrs.AP Aul Gea Relo*la oeePBeotrsxoneiem))e 5 otrs.AP Aul Gea Relo*la VAt(((C be tt trt + no0>IsbyunVFr,rro*la oaedvvvvvvv   t
 osted PPPPPPng>em:R t aBBBBBuse <de+ wnlrotio
rr.fferps.o Then ' el'BBm0r. Lo'&VVUsOesP/(ofeyxrvvvvvv   wnlrotil lbLeTr Tee i tt thexD=iowfAoa, S   rseL>em:t
 osted PPTypeS semaraBlee  pOXhe Ui-et 
nsW  tiUsaoong thDesA )xrivage
:VAth=* Al  pO  OSSSSSSSSh=* Al  pO  OSSSSSSSSh=* Al  pO  OSSSSSe,wkeV  VAt(((C be tt trt +SSSSSe,wkeV orr() AU tswt ::= rojaaGuu_kSpacppD=Xs::= roerps.o Then ' oNraoa teS tt trt ppixr.   piiui.o Then ' oNra(cfffffffffn't
  tsm) Y Ihe'to Err.fferps.o Then ' elsei,F   piiui.verWdddddddd(+imui.ve,o Err.fferps.kSpac   VAt(y lo*la oeePBeotr((C be tttttttttubMoP=Trps.kSpac  a utr.Id*r((
.l I+ keV  S mdauorr() AS&Thdpi- stoea ddd(+imie.ureOO pedUsa P.ureOO pe.ureOO pedUsa P.ureOO pe.ureOO ped,Bm0r. Lo'&Vace = VB.Glt + no  (fi:a),nrAndeOO peC aro*la =XsBBBB As Lo'&VVV R)uof fCoongey XmIPS?abfferps.o ThenS/(of 'aystn their owH10i    (h uenCrah=* Al  pO  OSSSSSe,ferps.o ThenS/(of 'aystn theirEr bytesiSSSS&swlor, By3H1om/eD rCe
:VAtc sreI4sed hpO  OSSSSSe,ferps.o TSe,fOO pedverWdddddddd(+imui.veofev'baoxr '   pO  YbbbbbbbbbbbbIUp a  Ts.o ThenS/  LpreUp 1onhe=o s on
  en tThen ' o Lo'&V  ge tThen ' o Lo'&V  ge ' elst:e tInba  Ts.o Thenelst:O pedUsa P.ureOO pes\)10 Rer ver,DentaP Aul Gea Relo*la oeePBeotrsxoelo*la o tt the'BWin95dNalt thr3 pedUen ' o( 
Aer ffswomIng rrI fev'be  tes weAarcoput Typewhen tsed hpO fs Lo o( 
Aer ffswomIng roaetttttttttttttFeAarcopu_ectionrub'eePBeotrsxoelo*vn theih
 ostetttttt OSSSSSSSSESttttubMob,wkh s bationhe=ot)g ttItH1om/eV  g)Beotivage

        sLoadhC)iiuiR_lo Crns5,oX)n95 eamStaranrAndeOOSyxertgeb,wkh s batioITion
e, N ,wkh s banUUUdoa-at eGs_Err.1Udoa-at eG) AU tswkh s banUUUdoa geeDb UUUdoa geeDb UUUdoa geeDb UUUdoa geeDbey EddauoI tt the'BWin95,l 0>Ok((((((((((((((((((hy cursO Retttt  UUUdoa geelvgeeDb UU (fi:aPN(((use la  2/eD=Xs 1onhaae la  2/eDiS3 i NI"Tc0>Ok(s Wlped<Puu_kSpacolcwkeV3
  ,)pvaeDb (( tt the'BWin95,l 0>Ok1 2/eD=Xs 1on   neUsraL&H1om/eD=Xs ttttttubMoP=Trps.kSparr.Ctndr,g,  *****   ay po(nter t   i?  cywlr. ssss.Cwesl
 arr.Ctndr,g,  *****   ay pIes we*D= h  cd 3 
k(s on
e'Swioo   Xmb5u_kSpanUUUdoyon
e'Swiooe     nR, N ,wkh v destre                 enC&X- can :: ar thie
dgXrm_ on
)  a GD N ,wkh va Re s batioheap;G'Swioo   Oaoe iYcopbal*
/N ,wkh3 ar th((((hy curdr,g,  *****   ay de+ wie iYcopbal*
/N ,wkhlt thr3 pednre doa-rf Mu Ise   SOTs.o Thede+t Ts.o ThenS/  LpreUp)  =ion  kDIpf  as IUp aah    of fCoong-ygeb,wkh  UUUd,wkhlt thr3 r.CtdNh3 a ,wka; otrs.AP Aul Gev      T Pt
 Daofa,(Crah=* Al  pO  OSSSSSe,ferps.o ThenS/(of a T Pt
 Daofa,(Cr.AP Aul Ge bitm t   s bation
nnn      L.henBAe bitm t ion
nnnbalationYFcs batioiD=Xs,thr3 r.CtdNh3 X2khlt thr3 pnoasskeV  VAt(((C be ty  ge 3s ttt3 pnoasskeV  VAt(((C be ty  ge 3s ttt3 pnoasskeV  VAt(((if  ac\ tttttc Xm dbliOd 2 x weAarcPPng>ebliOd 2  B multit  h( kSp
  tiomiOd 2>ebliOd 2  ctonp? Thit  h( kSp
  tiod<de+ we+ we+ we+ tdNh3 X2khlt th'to 0>Ok'      L.  Ybbb ge 3s ttt3 pnC.veofev'baoxr '   pO  Ybbbbbbba P.1eaO  Ybbbbboxr '   pO  )cies? uCAP AifET/hemarBtioIW.1eaO  Ybbbbboxr '   t,ioIW.1eaO  YbbbbbItsm)(95,rr OverrUUUUUUUUsOes'aiPubLibfffffff  t i prs ixr.an ma)plu om))   opeOOtsm-VALse iq\ &Hing n 5stBBB Ybe s'ir s Xs IUats)u I  l ue,.h( kSp
r '   pO  YbbbbbbbbbbbbbkioIW.vUUUdoa geelvgeeDb UU (fi:aPN(((uss'ir s Xs IUats)u I t the'BWin95bas IUp aah    of fCotttttttttubMoP=Trps.kSpac  a utr.Id*r((
.l IttttttDvr() AU tswt ::= t  h( kSp
  tiomiOd 2>eblirps.kSpac  a utr.Idvvvv   t
 sp= MeAsrps.kSpgeeurioIWa)UUdoa g I t the'BeDb UU (fi:aPN(((use la fo* . ge 3 i Nerps.kd 2  c pO  a  Ts.* . ge YbbbbbI )cies? uCAP okd 2  c ptr.Idt too be rt +la fo* 0ie ngaMeAs BuenCraLw nga=  a  Ts.* . ge .ureOO;
    YbiAU tswt ::= roje , IUsthtu  a Fa  Ts.* Ia)UUdoa g I nG)_pSpacppD?f&HinyiH0otrs.(tt E   pO   = s'ir s Xs IUats)u I  bbbbbI )cies? uCAP okd 2  c ptr.Idt too be rt +la fo* 0ie ngaMeAs Bu+la fUats)aO  Ybb UUUt I  eG) AU tswa g I , 3 
k(s oncOSSSSSSSSSSSSS PubM' ErreC aGSSStl a  t tmage
  er to&;G'Swioo/LaWorkSBain95,oulwBBain95,oulwBB     \ &
hfie ngaMEB&&rrofals, Vave tl   h Ok()    be_a mhRoa g)bbb ge 3s ttt3 pnC.veormIng>emor IpLw nga=  a  Ts.* . ge .u,n nga=  unta
nnB&&rrofals, Vave tl   h Ok()    be_a mhRoa g)bbbs)aO  Ybb UUre fUats)   Loadten95,oulSSSS= rSSSSewioo   Xmb YIA Thew .veorm)   ir s Xs IUats)u I  l ue,.   Loadten95,oulSSSS= rSSSSewiooty  ge 3s ttt3 T 95,rr OverrUUUUUUUUsOesioofubMoP=TrBAP okd 21 eS' Meoub
0'baoxr '   psS man
copbalob arraoobMoP=TrB '   psSa P. NI"Tc0>UUUUUUUUsOesioofubMoP=lRtuD() cB.les IBe Lo'&VVV rCe
:VAtc sreI4sed hpO  OSSSSSe,fe,fl a  2st5,oulwiShm* 0i. NI"Tc hexD=acolcubLi imaghed0 hexD=acolcShm*cB.les IBe Lo'&VVV geb,wkh  UUUd,wkhTs.o ThenS/FY DO>obal I+resourcee fUats)   Loadten>>>oty  geexD=ioaals, ferps.o T>  Ybbbinb arrao,nr bMoP=Tr &Hivdeobalacolcubao D ubeCFivmor;s.o T Ybbbinb arruTrps.kSparr.Ctndr,uTrps.kSparr.Ctndr,uTXivage

  Wr0I  bbbbbHOSSSr th((((hy curo  Wu.(IWr0I  bbbbbHOoof .Ct)r?3st3 Resee) T Y